package com.galaxis.manatee.controller;

import com.galaxis.manatee.dao.ChuanyunProjectAttachmentDao;
import com.galaxis.manatee.dao.ChuanyunUserDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectAttachmentDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserDO;
import com.galaxis.manatee.util.ExcelUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

@RestController
@Slf4j
public class ExcelController {

    private final ChuanyunUserDao chuanyunUserDao;
    private final ChuanyunProjectAttachmentDao chuanyunProjectAttachmentDao;

    public ExcelController(ChuanyunUserDao chuanyunUserDao, ChuanyunProjectAttachmentDao chuanyunProjectAttachmentDao) {
        this.chuanyunUserDao = chuanyunUserDao;
        this.chuanyunProjectAttachmentDao = chuanyunProjectAttachmentDao;
    }


    @ResponseBody
    @RequestMapping(value = "/test/exportExcel", method = RequestMethod.GET)
    public void exportExcel(HttpServletRequest request, HttpServletResponse response) throws Exception {
        List<ChuanyunProjectAttachmentDO> list = chuanyunProjectAttachmentDao.findAll();
        String[] title = {"项目名称", "项目编号", "物料名称", "图片", "英文名","零件归属","数量","单位","未税单价","易耗件","系数","汇率","总价","备注"};
        String filename = "testRecord.xls";
        String sheetName = "sheet1";
        String[][] content = new String[list.size()][20];
        try {
            for (int i = 0; i < list.size(); i++) {
                content[i][0] = list.get(i).getProjectName();
                content[i][1] = list.get(i).getProjectCode();
                content[i][2] = list.get(i).getMaterialName();
                content[i][3] = list.get(i).getEnclosureId();
                content[i][4] = list.get(i).getEnglishName();
                content[i][5] = list.get(i).getPartsOwnership();
                content[i][6] = String.valueOf(list.get(i).getQty());
                content[i][7] = list.get(i).getUnit();
                content[i][8] = String.valueOf(list.get(i).getUnitPricewithoutTax());
                content[i][9] = list.get(i).getConsumableParts();
                content[i][10] = String.valueOf(list.get(i).getCoefficient());
                content[i][11] = String.valueOf(list.get(i).getExchangeRate());
                content[i][12] = String.valueOf(list.get(i).getTotalPrice());
                content[i][13] = list.get(i).getRemarks();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        HSSFWorkbook wb = ExcelUtils.getHSSFWorkbook(sheetName, title, content, null);
        try {
            // 响应到客户端
            this.setResponseHeader(response, filename);
            OutputStream os = response.getOutputStream();
            wb.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 向客户端发送响应流方法
     *
     * @param response
     * @param fileName
     */
    public void setResponseHeader(HttpServletResponse response, String fileName) {
        try {
            try {
                fileName = new String(fileName.getBytes(), "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setContentType("application/vnd.ms-excel");
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
