package com.galaxis.manatee.controller;

import com.galaxis.manatee.service.LogCheckService;
import com.galaxis.manatee.service.LogService;
import com.galaxis.manatee.service.LogStandardService;
import com.galaxis.manatee.service.LogUpdateService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/10/27 11:39 上午
 */
@Slf4j
@RestController
public class TestController {

    private final LogUpdateService logUpdateService;
    private final LogStandardService logStandardService;
    private final LogCheckService logCheckService;
    private final LogService logService;

    public TestController(LogUpdateService logUpdateService, LogStandardService logStandardService, LogCheckService logCheckService, LogService logService) {
        this.logUpdateService = logUpdateService;
        this.logStandardService = logStandardService;
        this.logCheckService = logCheckService;
        this.logService = logService;
    }

    @GetMapping("/test/helloGet")
    public String testGet() {
        return "hello";
    }

    /**
     * 标准化所有周工时
     */
    @GetMapping("/test/api/standardAllWeekMemberHour")
    public void standardAllWeekMemberHour(){
        logService.standardAllWeekMemberHour();
    }

    /**
     * 标准化所有月工时
     */
    @GetMapping("/test/api/standardAllSelfCost")
    public void standardAllSelfCost(){
        logService.standardAllSelfCost();
    }

    /**
     * 根据用户Id标准化日工时
     * @param userId    用户Id
     */
    @GetMapping("/test/api/standardSelfWorkHourByUserId")
    public void standardSelfWorkHourByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardSelfWorkHourByUserId(userId);
    }

    /**
     * 根据用户Id标准化周工时
     * @param userId    用户Id
     */
    @GetMapping("/test/api/standardWeekMemberHourByUserId")
    public void standardWeekMemberHourByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardWeekMemberHourByUserId(userId);
    }

    /**
     * 根据用户Id标准化月工时
     * @param userId    用户Id
     */
    @GetMapping("/test/api/standardSelfCostByUserId")
    public void standardSelfCostByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardSelfCostByUserId(userId);
    }

    /**
     * 更新所有每天工时
     */
    @GetMapping("/test/api/updateAllChuanyunSelfWorkHour")
    public void updateAllChuanyunSelfWorkHour() {
        logService.updateAllChuanyunSelfWorkHour();
    }

    /**
     * 更新所有每周工时
     */
    @GetMapping("/test/api/updateAllChuanyunWeekMemberHour")
    public void updateAllChuanyunWeekMemberHour() {
        logService.updateAllChuanyunWeekMemberHour();
    }

    /**
     * 更新所有每月工时
     */
    @GetMapping("/test/api/updateAllChuanyunSelfCost")
    public void updateAllChuanyunSelfCost() {
        logService.updateAllChuanyunSelfCost();
    }

    /**
     * 根据用户Id更新每天工时方法
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/updateChuanyunSelfWorkHourListByUserId")
    public void updateChuanyunSelfWorkHourListByUserId(@RequestParam("userId") String userId) {
        logUpdateService.updateChuanyunSelfWorkHourListByUserId(userId);
    }

    /**
     * @param userId 用户Id
     */
    @GetMapping("/test/api/updateChuanyunWeekMemberHourListByUserId")
    public void updateChuanyunWeekMemberHourListByUserId(@RequestParam("userId") String userId) {
        logUpdateService.updateChuanyunWeekMemberHourListByUserId(userId);
    }

    /**
     * @param userId 用户Id
     */
    @GetMapping("/test/api/updateChuanyunSelfCostListByUserId")
    public void updateChuanyunSelfCostListByUserId(@RequestParam("userId") String userId) {
        logUpdateService.updateChuanyunSelfCostListByUserId(userId);
    }

    /**
     * 根据用户Id核查工时信息
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/checkWorkHourByUserId")
    public void checkWorkHourByUserId(@RequestParam("userId") String userId) {
        long start = System.currentTimeMillis();
        try {
            //核对原始数据和标准化每日工时
            logCheckService.checkDayWorkHour(userId);
            logCheckService.checkWeekWorkHour(userId);
            logCheckService.checkMonthWorkHour(userId);
        } catch (Exception e) {
            log.error(userId);
            log.error(e.getMessage());
        }
        log.info("修改" + userId + "工时信息耗时" + (System.currentTimeMillis() - start));

    }

}


