package com.galaxis.manatee.entity.chuanyun.data.object;


import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @author kxuan
 */
@Slf4j
@Data
@Entity(name = "CHUANYUN_SELF_WORK_HOUR")
@JsonIgnoreProperties(ignoreUnknown = true)
@Table(indexes = {@Index(name="project_user_hour", columnList="projectId,userId,standardWorkHour", unique=false)})
public class ChuanyunSelfWorkHourDO {

    @Id
    @GeneratedValue(generator = "idGeneratorUser")
    @GenericGenerator(name ="idGeneratorUser" ,strategy="com.galaxis.capsule.util.GalaxisIdGenerator")
    private Long objectId;

    private String userId;

    private String projectType;

    private String projectId;

    private String departmentId;

    private String departmentName;

    /**
     * 流程状态
     */
    private String status;

    @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    private LocalDateTime dayLogDate;

    private BigDecimal standardWorkHour;

    /**
     * 根据ChuanyunWorkHourDO对象实例化ChuanyunSelfWorkHourDO对象
     * @param projectType 由于日志原始数据可能出现项目类型错误的情况，需要单独为projectType赋值
     * @param chuanyunWorkHourDO 从日志原始数据中统计出来的人员单日项目工时对象
     * @return  更新了ChuanyunWorkHourDO对象中数据的ChuanyunSelfWorkHourDO对象
     */
    public static ChuanyunSelfWorkHourDO fromChuanyunWorkHourDO(ChuanyunWorkHourDO chuanyunWorkHourDO,String projectType){
        ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO = new ChuanyunSelfWorkHourDO();
        chuanyunSelfWorkHourDO.setStatus(chuanyunWorkHourDO.getStatus());
        chuanyunSelfWorkHourDO.setDepartmentName(chuanyunWorkHourDO.getDepartmentName());
        chuanyunSelfWorkHourDO.setUserId(chuanyunWorkHourDO.getUserId());
        //2021年3月3日添加,使用ChuanyunGroupProject中实际的项目类型来替代日志中的项目类型，防止出现项目类型错误的情况
        chuanyunSelfWorkHourDO.setProjectType(projectType);
        if (ChuanYunConstant.PRODUCTION_PROJECT.equals(chuanyunWorkHourDO.getProjectType())) {
            chuanyunSelfWorkHourDO.setProjectId(ChuanYunConstant.PRODUCTION_PROJECT_ID);
        } else {
            chuanyunSelfWorkHourDO.setProjectId(chuanyunWorkHourDO.getProjectId());
        }
        chuanyunSelfWorkHourDO.setDayLogDate(chuanyunWorkHourDO.getDayLogDate());
        return chuanyunSelfWorkHourDO;
    }
}
