package com.galaxis.manatee.entity.chuanyun.dto;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.galaxis.manatee.constant.StringConstant;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * 氚云批量查询过滤条件,氚云技术人员建议查询全部数据，自己进行筛选
 * @author zcj
 * @version 0.1
 * @date 2020/2/5 10:01 下午
 */
@Slf4j
@Getter
@Setter
public class Filter {

    public static final String AND="And";

    public static final String OR="Or";

    /**
     *
     * @param fromRowNumber 从第几行开始
     * @param toRowNumber   到第几行
     * @param requireCount  是否需要总数
     * @return  Filter对象
     */
    public  static Filter instance(Integer fromRowNumber,Integer toRowNumber,Boolean requireCount){
        return new Filter(fromRowNumber,toRowNumber,requireCount,new String[0],new String[0],AND,new ArrayList<>());
    }

    /**
     *
     * @param fromRowNumber 从第几行开始
     * @param toRowNumber   到第几行
     * @param requireCount  是否需要总数
     * @param type          条件逻辑运算
     * @param matchers      过滤条件字符串数组
     *                      XXXX_YY,ZZZZ
     *                      XXXX为字段名
     *                      YY为判断条件
     *                      ZZZZ为查询值
     * @return  Filter对象
     */
    public  static Filter instance(Integer fromRowNumber,Integer toRowNumber,Boolean requireCount,String type,List<String> matchers){
        return new Filter(fromRowNumber,toRowNumber,requireCount,new String[0],new String[0],type,matchers);
    }

    /**
     * 分页查询，从第几条开始
     */
    @JsonProperty(value = "FromRowNum")
    private Integer fromRowNumber;
    /**
     * 分页查询，第几条结束
     */
    @JsonProperty(value = "ToRowNum")
    private Integer toRowNumber;
    /**
     * 查询的总行数
     */
    @JsonProperty(value = "RequireCount")
    private Boolean requireCount;
    /**
     * 返回的字段，不填返回所有
     */
    @JsonProperty(value = "ReturnItems")
    private String[] returnItem;
    /**
     * 排序字段，目前不支持使用，默认置空
     */
    @JsonProperty(value = "SortByCollection")
    private String[] sortByCollection;
    /**
     * 查询条件
     */
    @JsonProperty(value = "Matcher")
    private Matcher matcher;


    private Filter(Integer fromRowNumber, Integer toRowNumber, Boolean requireCount, String[] returnItem, String[] sortByCollection,String type, List<String> matchersString) {
        this.fromRowNumber = fromRowNumber;
        this.toRowNumber = toRowNumber;
        this.requireCount = requireCount;
        this.returnItem = returnItem;
        this.sortByCollection = sortByCollection;
        this.matcher = new Matcher(type,stringToMatchers(matchersString));
    }

    private List<Matchers> stringToMatchers(List<String> matchersString){
        List<Matchers> matchersList=new ArrayList<>();
        matchersString.forEach(string->{
            try{
                var keyValue=string.split(StringConstant.COMMA);
                var keyOperator=keyValue[0].split(StringConstant.UNDER_LINE);
                var matchers=new Matchers("Item",keyOperator[0],keyOperator[1],keyValue[1]);
                matchersList.add(matchers);
            }catch (Exception e){
                log.warn("查询条件解析异常");
                log.warn(string);
                e.printStackTrace();
            }
        });
        return matchersList;
    }

    @Getter
    @Setter
    public static class Matcher{
        /**
         * 条件连接词
         * 默认值：and  or
         */
        @JsonProperty(value = "Type")
        private String type;

        /**
         * 判断条件
         */
        @JsonProperty(value = "Matchers")
        private List<Matchers>matchers;

        public Matcher(String type, List<Matchers> matchers) {
            this.type = type;
            this.matchers = matchers;
        }
    }

    @Getter
    @Setter
    public static class Matchers{
        /**
         * 拼接条件时 and or
         * 设置查询条件时 item
         */
        @JsonProperty("Type")
        private String type;
        /**
         * 字段名
         */
        @JsonProperty("Name")
        private String name;
        /**
         * Operator运算符：
         * 0=大于，
         * 1=大于等于，
         * 2=等于
         * 3=小于等于，
         * 4=小于，
         * 5=不等于，
         * 6=在某个范围内，
         * 7=不在某个范围内。
         */
        @JsonProperty("Operator")
        private String operator;

        /**
         * 数值
         */
        @JsonProperty("Value")
        private String value;

        @JsonProperty("Matchers")
        private List<Matchers> matchers;

        public Matchers(String type, String name, String operator, String value) {
            this.type = type;
            this.name = name;
            this.operator = operator;
            this.value = value;
        }
    }
}
