package com.galaxis.manatee.manager;

import com.galaxis.capsule.dto.*;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectCashFlowDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunPurCostDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunReimbursementDO;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.entity.chuanyun.dto.MaterialDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/8 4:22 下午
 */
@FeignClient(name = "claw",url = "${claw.url}")
public interface ClawFeign {
    /**
     * 查询物料信息
     * @param map   查询参数
     * @return  请求结果
     */
    @GetMapping("/material/findAllByParameters")
    PageDTO<MaterialDTO> findMaterialPageable(@RequestParam Map<String,String> map);

    /**
     * 查询项目信息
     * @param map   查询参数
     * @return  请求结果
     */
    @GetMapping("/chuanyunProject/findAllByParameters")
    PageDTO<ChuanyunProjectDTO> findChuanyunProjectPageable(@RequestParam Map<String,String> map);

    /**
     * 查询项目材料成本
     * @param map
     * @return
     */
    @GetMapping("/cashflow/findAllByParameters")
    PageDTO<ChuanyunProjectCashFlowDTO> findChuanyunProjectCashFlow(@RequestParam Map<String,String> map);

    /**
     * 查询项目收付款
     * @param map
     * @return
     */
    @GetMapping("/purcost/findAllByParameters")
    PageDTO<ChuanyunPurCostDTO> findChuanyunPurCost(@RequestParam Map<String,String> map);

    /**
     * 获取供应商
     * @param map
     * @return
     */
    @GetMapping("/supplier/findAllByParameters")
    PageDTO<ChuanyunSupplierDTO> findChuanyunSupplier(@RequestParam Map<String,String> map);

    /**
     * 查询人力成本
     * @param map
     * @return
     */
    @GetMapping("/laborcost/findAllByParameters")
    PageDTO<ChuanyunLaborCostDTO> findChuanyunLaborCost(@RequestParam Map<String,String> map);

    /**
     * 查询设备配置信息
     * @param map
     * @return
     */
    @GetMapping("/deviceconfigure/findAllByParameters")
    PageDTO<ChuanyunDeviceConfigureDTO> findChuanyunDeviceConfigure(@RequestParam Map<String,String> map);

    /**
     * 查询设备信息
     * @param map
     * @return
     */
    @GetMapping("/device/findAllByParameters")
    PageDTO<ChuanyunDeviceDTO> findChuanyunDevice(@RequestParam Map<String,String> map);

    /**
     * 查询销售合同信息
     * @param map
     * @return
     */
    @GetMapping("/salescontract/findAllByParameters")
    PageDTO<ChuanyunSalesContractDTO> findChuanyunSalesContract(@RequestParam Map<String,String> map);

    /**
     * 查询销售合同报价信息
     * @param map
     * @return
     */
    @GetMapping("/salescontractoffer/findAllByParameters")
    PageDTO<ChuanyunSalesContractOfferDTO> findChuanyunSalesContractOffer(@RequestParam Map<String,String> map);

    /**
     * 查询销售合同回款明细
     * @param map
     * @return
     */
    @GetMapping("/salescollection/findAllByParameters")
    PageDTO<ChuanyunSalesCollectionDetailDTO> findChuanyunSalesCollectionDetail(@RequestParam Map<String,String> map);

    /**
     * 查询采购合同管理
     * @param map
     * @return
     */
    @GetMapping("/purcontract/findAllByParameters")
    PageDTO<ChuanyunPurContractDTO> findChuanyunPurContract(@RequestParam Map<String,String> map);

    /**
     * 根据合同号查询采购合同付款情况
     * @param docno
     * @return
     */
    @GetMapping("/purcontractdetail/findallbydocno")
    List<ChuanyunPurContractDetailDTO> findPurContractDetailByDocno(@RequestParam String docno);

    /**
     * 根据合同号查询采购明细
     * @param docno
     * @return
     */
    @GetMapping("/purcontractoffer/findallbydocno")
    List<ChuanyunPurContractOfferDTO> findPurContractOfferByDocno(@RequestParam String docno);

    /**
     * 根据合同号查询回款明细
     * @param docno
     * @return
     */
    @GetMapping("/salescollection/findallbydocno")
    List<ChuanyunSalesCollectionDetailDTO> findCollectionByDocno(@RequestParam String docno);

    /**
     * 根据合同号查询报价
     * @param docno
     * @return
     */
    @GetMapping("/salescontractoffer/findallbydocno")
    List<ChuanyunSalesContractOfferDTO> findAllByDocno(@RequestParam String docno);


    /**
     * 测试
     * @param chuanyunProjectDTO    保存DTO数据
     * @return  请求结果
     */
    @GetMapping("/chuanyunProject/save")
    ResponseEntity<ChuanyunProjectDTO> saveChuanyunProject(@RequestBody ChuanyunProjectDTO chuanyunProjectDTO);

    /**
     * 保存报销信息
     * @param chuanyunReimbursementDTO   报销数据
     * @return  保存结果
     */
    @PostMapping("/chuanyunReimbursement/save")
    ResponseEntity<U9ResponseDTO> saveReimbursement(@RequestBody ChuanyunReimbursementDTO chuanyunReimbursementDTO);

}
