package com.galaxis.manatee.service;

import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiProcessinstanceGetRequest;
import com.dingtalk.api.request.OapiProcessinstanceListidsRequest;
import com.dingtalk.api.response.OapiProcessinstanceGetResponse;
import com.dingtalk.api.response.OapiProcessinstanceListidsResponse;
import com.galaxis.manatee.constant.DingTalkConstant;
import com.galaxis.manatee.dao.ChuanyunAskForLeaveDao;
import com.galaxis.manatee.dao.ChuanyunBusinessEntertainmentDao;
import com.galaxis.manatee.dao.ChuanyunProjectChangeDao;
import com.galaxis.manatee.dao.DingTalkProcessInstanceDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunAskForLeaveDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunBusinessEntertainmentDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectChangeDO;
import com.galaxis.manatee.entity.ding.DingTalkProcessInstance;
import com.taobao.api.ApiException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

/**
 * 钉钉定时任务
 * @author zcj
 * @version 0.1
 * @date 2020/1/10 2:28 下午
 */
@Slf4j
@Service
public class DingTalkScheduledTask {

    /**
     * 审批Id列表客户端
     */
    private final DingTalkClient processInstanceListIdsClient;

    /**
     * 审批实例客户端
     */
    private final DingTalkClient processInstanceClient;

    /**
     * 钉钉审批实例保存方法
     */
    private final DingTalkProcessInstanceDao dingTalkProcessInstanceDao;

    /**
     * 钉钉请假数据同步到氚云
     */
    private final ChuanyunAskForLeaveDao chuanyunAskForLeaveDao;

    private final ChuanyunBusinessEntertainmentDao chuanyunBusinessEntertainmentDao;
    private final ChuanyunProjectChangeDao chuanyunProjectChangeDao;

    public DingTalkScheduledTask(DingTalkClient processInstanceListIdsClient, DingTalkClient processInstanceClient,
                                 ChuanyunAskForLeaveDao chuanyunAskForLeaveDao, DingTalkProcessInstanceDao dingTalkProcessInstanceDao, ChuanyunBusinessEntertainmentDao chuanyunBusinessEntertainmentDao, ChuanyunProjectChangeDao chuanyunProjectChangeDao) {
        this.processInstanceListIdsClient = processInstanceListIdsClient;
        this.processInstanceClient = processInstanceClient;
        this.dingTalkProcessInstanceDao = dingTalkProcessInstanceDao;
        this.chuanyunAskForLeaveDao = chuanyunAskForLeaveDao;
        this.chuanyunBusinessEntertainmentDao = chuanyunBusinessEntertainmentDao;
        this.chuanyunProjectChangeDao = chuanyunProjectChangeDao;
    }

    /**
     * 每个小时定期更新出差申请数据
     * 出差申请信息暂时不进行更新
     */
//    @Scheduled(fixedDelay = 3600000L,initialDelay = 10000L)
    void updateBusinessTripProcessInstanceIdList() throws ApiException {
        updateProcessInstanceIdList(DingTalkConstant.PROCESS_CODE_BUSINESS_TRIP);
    }



    /**
     * 每小时根据出差申请Id跟新出差申请信息。
     * 出差申请信息暂时不进行更新
     */
//    @Scheduled(fixedDelay = 3600000L,initialDelay = 10000L)
    void updateProcessInstanceList() throws ApiException {
         updateChuanyunAskForLeave(DingTalkConstant.PROCESS_CODE_ASKFORLEAVE);
    }

    /**
     * 业务招待申请更新
     * @throws ApiException
     */
//    @Scheduled(fixedDelay = 3600000L,initialDelay = 10000L)
    void updateChuanyunBusinessEntertainmentProcessInstanceList() throws ApiException {
        updateChuanyunBusinessEntertainment(DingTalkConstant.PROCESS_CODE_BUSINESSENTERTAINMENT);
    }

    /**
     * 每个小时定期更新项目实施变更申请数据
     */
//    @Scheduled(fixedDelay = 3600000L,initialDelay = 10000L)
    void updateProjectChangeProcessInstanceIdList() throws ApiException {
        updateChuanyunProjectChangeProcessInstanceIdList(DingTalkConstant.PROCESS_CODE_PROJECT_CHANGE);
    }


    private void updateChuanyunBusinessEntertainment(String businessTripProcessCode) throws ApiException {
        var flag=true;
        //单个请求
        var oApiProcessInstanceListIdsRequest=new OapiProcessinstanceListidsRequest();
        oApiProcessInstanceListIdsRequest.setProcessCode(businessTripProcessCode);
        oApiProcessInstanceListIdsRequest.setStartTime(LocalDateTime.now().minusDays(90).toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setEndTime(LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setCursor(0L);
        oApiProcessInstanceListIdsRequest.setSize(20L);
        while(flag){
            LocalDateTime start=LocalDateTime.now();
            OapiProcessinstanceListidsResponse oapiProcessinstanceListidsResponse=processInstanceListIdsClient.execute(oApiProcessInstanceListIdsRequest,DingTalkConstant.DING_TALK_TOKEN);
            List<ChuanyunBusinessEntertainmentDO> chuanyunBusinessEntertainmentDOS=new ArrayList<>();
            if(oapiProcessinstanceListidsResponse.getErrcode()!=0){
                log.info(oapiProcessinstanceListidsResponse.getErrmsg());
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    log.error(e.getMessage());
                }
                continue;
            }
            //逐个请求
            oapiProcessinstanceListidsResponse.getResult().getList().forEach(resultId->{
                Optional<DingTalkProcessInstance> instance=dingTalkProcessInstanceDao.findById(resultId);
                if(instance.isEmpty()){
                    OapiProcessinstanceGetRequest request=new OapiProcessinstanceGetRequest();
                    request.setProcessInstanceId(resultId);
                    try {
                        OapiProcessinstanceGetResponse oapiProcessinstanceGetResponse = processInstanceClient.execute(request, DingTalkConstant.DING_TALK_TOKEN);
                        List<OapiProcessinstanceGetResponse.FormComponentValueVo> formComponentValueVoList = oapiProcessinstanceGetResponse.getProcessInstance().getFormComponentValues();
                        ChuanyunBusinessEntertainmentDO chuanyunBusinessEntertainmentDO = ChuanyunBusinessEntertainmentDO.formBusinessEntertainmentDTO(formComponentValueVoList);
                        chuanyunBusinessEntertainmentDO.setBusinessId(oapiProcessinstanceGetResponse.getProcessInstance().getBusinessId());
                        chuanyunBusinessEntertainmentDO.setObjectId(UUID.randomUUID().toString());
                        ChuanyunBusinessEntertainmentDO oldChuanyunBusinessEntertainmentDO = chuanyunBusinessEntertainmentDao.findByBusinessId(oapiProcessinstanceGetResponse.getProcessInstance().getBusinessId());
                        if(oldChuanyunBusinessEntertainmentDO != null ) {
                            chuanyunBusinessEntertainmentDao.deleteById(oldChuanyunBusinessEntertainmentDO.getObjectId());
                        }
                        chuanyunBusinessEntertainmentDOS.add(chuanyunBusinessEntertainmentDO);
                    } catch (ApiException e) {
                        e.printStackTrace();
                    }
                }
            });

            //保存id列表和实例列表,如果有更新的内容
            chuanyunBusinessEntertainmentDao.saveAll(chuanyunBusinessEntertainmentDOS);
            log.info(businessTripProcessCode+"下一个cursor为"+oapiProcessinstanceListidsResponse.getResult().getNextCursor()+"保存成功"+"耗时"+ ChronoUnit.SECONDS.between(start,LocalDateTime.now())+"秒");
            flag=null!=oapiProcessinstanceListidsResponse.getResult().getNextCursor();
            if (flag){
                oApiProcessInstanceListIdsRequest.setCursor(oapiProcessinstanceListidsResponse.getResult().getNextCursor());
            }
        }
    }

    private void updateChuanyunAskForLeave(String businessTripProcessCode) throws ApiException {
            var flag=true;
            //单个请求
            var oApiProcessInstanceListIdsRequest=new OapiProcessinstanceListidsRequest();
            oApiProcessInstanceListIdsRequest.setProcessCode(businessTripProcessCode);
            oApiProcessInstanceListIdsRequest.setStartTime(LocalDateTime.now().minusDays(90).toInstant(ZoneOffset.UTC).toEpochMilli());
            oApiProcessInstanceListIdsRequest.setEndTime(LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli());
            oApiProcessInstanceListIdsRequest.setCursor(0L);
            oApiProcessInstanceListIdsRequest.setSize(20L);
            while(flag){
                LocalDateTime start=LocalDateTime.now();
                OapiProcessinstanceListidsResponse oapiProcessinstanceListidsResponse=processInstanceListIdsClient.execute(oApiProcessInstanceListIdsRequest,DingTalkConstant.DING_TALK_TOKEN);
                List<ChuanyunAskForLeaveDO> chuanyunAskForLeaveDoList=new ArrayList<>();
                if(oapiProcessinstanceListidsResponse.getErrcode()!=0){
                    log.info(oapiProcessinstanceListidsResponse.getErrmsg());
                    try {
                        Thread.sleep(5000);
                    } catch (InterruptedException e) {
                        log.error(e.getMessage());
                    }
                    continue;
                }
                //逐个请求
                oapiProcessinstanceListidsResponse.getResult().getList().forEach(resultId->{
                    Optional<DingTalkProcessInstance> instance=dingTalkProcessInstanceDao.findById(resultId);
                    if(instance.isEmpty()){
                        OapiProcessinstanceGetRequest request=new OapiProcessinstanceGetRequest();
                        request.setProcessInstanceId(resultId);
                        try {
                            OapiProcessinstanceGetResponse oapiProcessinstanceGetResponse = processInstanceClient.execute(request, DingTalkConstant.DING_TALK_TOKEN);
                            ChuanyunAskForLeaveDO chuanyunAskForLeaveDo = ChuanyunAskForLeaveDO.fromAskForLeaveDTO(oapiProcessinstanceGetResponse.getProcessInstance());
                            List<ChuanyunAskForLeaveDO>   existsAskForLeave = chuanyunAskForLeaveDao.findByTitleSQL(chuanyunAskForLeaveDo.getTitle());
                            if(existsAskForLeave != null && !existsAskForLeave.isEmpty()) {
                                chuanyunAskForLeaveDao.deleteById(existsAskForLeave.get(0).getObjectId());
                            }
                            chuanyunAskForLeaveDoList.add(chuanyunAskForLeaveDo);
                        } catch (ApiException e) {
                            e.printStackTrace();
                        }
                    }
                });

                //保存id列表和实例列表,如果有更新的内容
                chuanyunAskForLeaveDao.saveAll(chuanyunAskForLeaveDoList);
                log.info(businessTripProcessCode+"下一个cursor为"+oapiProcessinstanceListidsResponse.getResult().getNextCursor()+"保存成功"+"耗时"+ ChronoUnit.SECONDS.between(start,LocalDateTime.now())+"秒");
                flag=null!=oapiProcessinstanceListidsResponse.getResult().getNextCursor();
                if (flag){
                    oApiProcessInstanceListIdsRequest.setCursor(oapiProcessinstanceListidsResponse.getResult().getNextCursor());
                }
            }
    }

    private void updateProcessInstanceIdList(String businessTripProcessCode) throws ApiException {
        var flag=true;
        //单个请求
        var oApiProcessInstanceListIdsRequest=new OapiProcessinstanceListidsRequest();
        oApiProcessInstanceListIdsRequest.setProcessCode(businessTripProcessCode);
        oApiProcessInstanceListIdsRequest.setStartTime(LocalDateTime.now().minusDays(90).toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setEndTime(LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setCursor(0L);
        oApiProcessInstanceListIdsRequest.setSize(20L);
        while(flag){
            LocalDateTime start=LocalDateTime.now();
            OapiProcessinstanceListidsResponse oapiProcessinstanceListidsResponse=processInstanceListIdsClient.execute(oApiProcessInstanceListIdsRequest,DingTalkConstant.DING_TALK_TOKEN);
            List<DingTalkProcessInstance> dingTalkProcessInstanceList=new ArrayList<>();
            if(oapiProcessinstanceListidsResponse.getErrcode()!=0){
                log.info(oapiProcessinstanceListidsResponse.getErrmsg());
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    log.error(e.getMessage());
                }
                continue;
            }
            //逐个请求
            oapiProcessinstanceListidsResponse.getResult().getList().forEach(resultId->{
                Optional<DingTalkProcessInstance> instance=dingTalkProcessInstanceDao.findById(resultId);
                if(instance.isEmpty()){
                    //&&instance.get()==null
                    //淘宝SDK方法
                    OapiProcessinstanceGetRequest request=new OapiProcessinstanceGetRequest();
                    request.setProcessInstanceId(resultId);
                    DingTalkProcessInstance dingTalkProcessInstance=new DingTalkProcessInstance(resultId,businessTripProcessCode);
                    try {
                        OapiProcessinstanceGetResponse oapiProcessinstanceGetResponse = processInstanceClient.execute(request, DingTalkConstant.DING_TALK_TOKEN);
                        dingTalkProcessInstance.setProcessInstance(oapiProcessinstanceGetResponse.getProcessInstance());
                    } catch (ApiException e) {
                        e.printStackTrace();
                    }
                    //淘宝SDK方法结束
                    dingTalkProcessInstanceList.add(dingTalkProcessInstance);
                }
            });

            //保存id列表和实例列表
            dingTalkProcessInstanceDao.saveAll(dingTalkProcessInstanceList);
            log.info(businessTripProcessCode+"下一个cursor为"+oapiProcessinstanceListidsResponse.getResult().getNextCursor()+"保存成功"+"耗时"+ ChronoUnit.SECONDS.between(start,LocalDateTime.now())+"秒");
            flag=null!=oapiProcessinstanceListidsResponse.getResult().getNextCursor();
            if (flag){
                oApiProcessInstanceListIdsRequest.setCursor(oapiProcessinstanceListidsResponse.getResult().getNextCursor());
            }
        }
    }

    private void updateChuanyunProjectChangeProcessInstanceIdList(String businessTripProcessCode) throws ApiException {
        var flag=true;
        //单个请求
        var oApiProcessInstanceListIdsRequest=new OapiProcessinstanceListidsRequest();
        oApiProcessInstanceListIdsRequest.setProcessCode(businessTripProcessCode);
        oApiProcessInstanceListIdsRequest.setStartTime(LocalDateTime.now().minusDays(90).toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setEndTime(LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setCursor(0L);
        oApiProcessInstanceListIdsRequest.setSize(20L);
        while(flag){
            LocalDateTime start=LocalDateTime.now();
            OapiProcessinstanceListidsResponse oapiProcessinstanceListidsResponse=processInstanceListIdsClient.execute(oApiProcessInstanceListIdsRequest,DingTalkConstant.DING_TALK_TOKEN);
            List<ChuanyunProjectChangeDO> chuanyunProjectChangeDOList=new ArrayList<>();
            if(oapiProcessinstanceListidsResponse.getErrcode()!=0){
                log.info(oapiProcessinstanceListidsResponse.getErrmsg());
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    log.error(e.getMessage());
                }
                continue;
            }
            //逐个请求
            oapiProcessinstanceListidsResponse.getResult().getList().forEach(resultId->{
                Optional<DingTalkProcessInstance> instance=dingTalkProcessInstanceDao.findById(resultId);
                if(instance.isEmpty()){
                    //&&instance.get()==null
                    //淘宝SDK方法
                    OapiProcessinstanceGetRequest request=new OapiProcessinstanceGetRequest();
                    request.setProcessInstanceId(resultId);
                    try {
                        OapiProcessinstanceGetResponse oapiProcessinstanceGetResponse = processInstanceClient.execute(request, DingTalkConstant.DING_TALK_TOKEN);
                        ChuanyunProjectChangeDO chuanyunProjectChangeDO = ChuanyunProjectChangeDO.fromProjectChangeDTO(oapiProcessinstanceGetResponse.getProcessInstance());
                        List<ChuanyunProjectChangeDO>   existsProjectChange = chuanyunProjectChangeDao.findByTitleSQL(chuanyunProjectChangeDO.getTitle());
                        if(existsProjectChange != null && !existsProjectChange.isEmpty()) {
                            chuanyunProjectChangeDao.deleteById(existsProjectChange.get(0).getObjectId());
                        }
                        chuanyunProjectChangeDOList.add(chuanyunProjectChangeDO);
                    } catch (ApiException e) {
                        e.printStackTrace();
                    }
                }
            });

            //保存id列表和实例列表
            chuanyunProjectChangeDao.saveAll(chuanyunProjectChangeDOList);
            log.info(businessTripProcessCode+"下一个cursor为"+oapiProcessinstanceListidsResponse.getResult().getNextCursor()+"保存成功"+"耗时"+ ChronoUnit.SECONDS.between(start,LocalDateTime.now())+"秒");
            flag=null!=oapiProcessinstanceListidsResponse.getResult().getNextCursor();
            if (flag){
                oApiProcessInstanceListIdsRequest.setCursor(oapiProcessinstanceListidsResponse.getResult().getNextCursor());
            }
        }
    }
}
