package com.galaxis.manatee.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.ChuanyunSelfCostDao;
import com.galaxis.manatee.dao.ChuanyunSelfWorkHourDao;
import com.galaxis.manatee.dao.ChuanyunWeekMemberHourDao;
import com.galaxis.manatee.dao.ChuanyunWorkHourDao;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zcj
 * @version 0.1
 * @date 2021/3/4 4:27 上午
 */
@Slf4j
@Service
public class LogCheckService {
    private final ObjectMapper objectMapper = new ObjectMapper();
    private final ChuanyunWorkHourDao chuanyunWorkHourDao;
    private final ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunWeekMemberHourDao chuanyunWeekMemberHourDao;
    private final ChuanyunSelfCostDao chuanyunSelfCostDao;

    public LogCheckService(ChuanyunWorkHourDao chuanyunWorkHourDao, ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao, ChuanYunManager chuanYunManager, ChuanyunWeekMemberHourDao chuanyunWeekMemberHourDao, ChuanyunSelfCostDao chuanyunSelfCostDao) {
        this.chuanyunWorkHourDao = chuanyunWorkHourDao;
        this.chuanyunSelfWorkHourDao = chuanyunSelfWorkHourDao;
        this.chuanYunManager = chuanYunManager;
        this.chuanyunWeekMemberHourDao = chuanyunWeekMemberHourDao;
        this.chuanyunSelfCostDao = chuanyunSelfCostDao;
    }

    /**
     * 根据当前userId核对以下内容：
     * 日志原始数据每日各个项目工时信息（可使用ChuanyunWorkHourDao中方法进行查询），与系统中保存每日工时（ChuanyunSelfWorkHourDO）进行比较
     * 完成核对后，删除数据库中多余的每周工时
     * 完成核对后，删除氚云中中多余的每周工时
     *
     * @param userId 用户Id
     * @throws Exception 用户Id为空异常
     */
    public void checkDayWorkHour(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户Id为空");
        }
        long start = System.currentTimeMillis();
        log.info("开始核对日志原始工时与标准化工时");
        //根据日志原始数据获取单个人的所有工时记录
        List<ChuanyunWorkHourDO> chuanyunWorkHourList = chuanyunWorkHourDao.getSelfWorkHour(userId);
        //根据标准化工时获取单个人的标准化工时记录
        List<ChuanyunSelfWorkHourDO> chuanyunSelfWorkHourList = chuanyunSelfWorkHourDao.findByUserId(userId);
        //删除多余每日工时
        compareDayWorkHour(chuanyunWorkHourList, chuanyunSelfWorkHourList);
        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 根据当前userId核对以下内容：
     * 标准化每日工时统计每周工时的结果（可使用ChuanyunWeekMemberHourDao中方法进行查询），与系统中保存每周工时(ChuanyunWeekMemberHourDO)进行比较
     * 完成核对后，更新或删除数据库中多余的每周工时
     * 完成核对后，更新或删除氚云中中多余的每周工时
     *
     * @param userId 用户Id
     * @throws Exception 用户Id为空异常
     */
    public void checkWeekWorkHour(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户Id为空");
        }
        long start = System.currentTimeMillis();
        log.info("开始核对标准化周工时");
        List<ChuanyunWeekMemberHourDO> list = chuanyunWeekMemberHourDao.getWeekMemberHourByUserId(userId);
        List<ChuanyunWeekMemberHourDO> oldlist = chuanyunWeekMemberHourDao.findByUserId(userId);
        Map<String, Integer> map = new HashMap<>(list.size()+oldlist.size());

        if (list.size()!=oldlist.size()){
            for (ChuanyunWeekMemberHourDO chuanyunWeekMemberHourDO : list){
                map.put(chuanyunWeekMemberHourDO.getUserId()+chuanyunWeekMemberHourDO.getProjectId()+chuanyunWeekMemberHourDO.getProjectType()+chuanyunWeekMemberHourDO.getYear()+chuanyunWeekMemberHourDO.getWeek(),1);
            }
            for (ChuanyunWeekMemberHourDO chuanyunWeekMemberHourDO : oldlist){
//                if (!chuanyunWeekMemberHourDO.getProjectType().equals("生产项目")){
                    Integer count = map.get(chuanyunWeekMemberHourDO.getUserId()+chuanyunWeekMemberHourDO.getProjectId()+chuanyunWeekMemberHourDO.getProjectType()+chuanyunWeekMemberHourDO.getYear()+chuanyunWeekMemberHourDO.getWeek());
                    if (count==null){
                        chuanyunWeekMemberHourDao.delete(chuanyunWeekMemberHourDO);
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000001_2,"+chuanyunWeekMemberHourDO.getProjectId());
                        matchers.add("F0000002_2,"+chuanyunWeekMemberHourDO.getUserId());
                        matchers.add("F0000006_2,"+chuanyunWeekMemberHourDO.getWeek());
                        matchers.add("F0000008_2,"+chuanyunWeekMemberHourDO.getYear());
                        var filter = Filter.instance(0, 1, true, "And", matchers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectMemberWeekHourDO.SCHEMA_CODE, filter);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunProjectMemberWeekHourDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            chuanyunSaveDTO = chuanYunManager.delete(ChuanyunProjectMemberWeekHourDO.SCHEMA_CODE, result.get(0).getObjectId());
                        }
                        log.info(chuanyunWeekMemberHourDO.getUserId()+"删除项目"+chuanyunWeekMemberHourDO.getProjectId()+"时间为"+chuanyunWeekMemberHourDO.getYear()+"年"+chuanyunWeekMemberHourDO.getWeek()+"周工时");
                    }
//                }
            }
        }
        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 根据当前userId核对以下内容：
     * 标准化每日工时统计每月工时的结果（可用ChuanyunSelfWorkHourDao中的方法），与系统中保存每月工时(ChuanyunSelfCostDO)进行比较
     * 完成核对后，更新或删除数据库中多余的每周工时
     * 完成核对后，更新或删除氚云中中多余的每周工时
     *
     * @param userId 用户Id
     * @throws Exception 用户Id为空异常
     */
    public void checkMonthWorkHour(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户Id为空");
        }

        long start = System.currentTimeMillis();
        log.info("开始核对标准化月工时");
        List<ChuanyunSelfWorkHourDO> list = chuanyunSelfWorkHourDao.getMonthProjectHour(userId);
        List<ChuanyunSelfCostDO> costDOList = chuanyunSelfCostDao.findByUserId(userId);
        if (list.size()!=costDOList.size()){
            Map<String, Integer> map = new HashMap<>(list.size()+costDOList.size());
            for (ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO : list){
                map.put(chuanyunSelfWorkHourDO.getUserId()+chuanyunSelfWorkHourDO.getProjectId()+chuanyunSelfWorkHourDO.getProjectType()+chuanyunSelfWorkHourDO.getDayLogDate().withDayOfMonth(1),1);
            }
            for (ChuanyunSelfCostDO chuanyunSelfCostDO : costDOList){
//                if (!chuanyunSelfCostDO.getProjectType().equals("生产项目")){
                    Integer count = map.get(chuanyunSelfCostDO.getUserId()+chuanyunSelfCostDO.getProjectId()+chuanyunSelfCostDO.getProjectType()+chuanyunSelfCostDO.getDayLogDate().withDayOfMonth(1));
                    if (count==null){
                        chuanyunSelfCostDao.delete(chuanyunSelfCostDO);
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000003_2,"+chuanyunSelfCostDO.getProjectId());
                        matchers.add("F0000002_2,"+chuanyunSelfCostDO.getUserId());
                        matchers.add("F0000004_2,"+chuanyunSelfCostDO.getDayLogDate().plusDays(1).withDayOfMonth(1));
                        var filter = Filter.instance(0, 1, true, "And", matchers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMemberMonthCostDO.SCHEMA_CODE, filter);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunMemberMonthCostDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            chuanyunSaveDTO = chuanYunManager.delete(ChuanyunMemberMonthCostDO.SCHEMA_CODE, result.get(0).getObjectId());
                        }
                        log.info(chuanyunSelfCostDO.getUserId()+"删除项目"+chuanyunSelfCostDO.getProjectId()+"时间为"+chuanyunSelfCostDO.getDayLogDate()+"月工时");
                    }
//                }
            }
        }
        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 根据projectId核对一下内容：
     * 每日工时统计的每个项目的总工时（可用ChuanyunProjectCostDao中方法）,与系统中保存的每个项目工时进行比较
     * 完成核对后，更新数据库中多余的每个项目工时
     * 完成核对后，更新氚云中多余的每个项目工时
     *
     * @param projectId 项目ID
     * @throws Exception 项目Id为空异常
     */
    public void checkProjectWorkHour(String projectId) throws Exception {
        //@TODO
        //***
        //@TODO
        updateChuanyunProjectWorkHour();
        updateManateeProjectWorkHour();
    }
//
//    /**
//     * 更新氚云每周工时数据
//     */
//    private void updateChuanyunWeekWorkHour() {
//        //@TODO
//    }
//
//    /**
//     * 更新Manatee数据库每周工时数据
//     */
//    private void updateManateeWeekWorkHour() {
//        //@TODO
//    }
//
//    /**
//     * 更新氚云每月工时数据
//     */
//    private void updateChuanyunMonthWorkHour() {
//        //@TODO
//    }
//
//    /**
//     * 更新Manatee数据库每月工时数据
//     */
//    private void updateManateeMonthWorkHour() {
//        //@TODO
//    }
//
    /**
     * 更新氚云每个项目时数据
     */
    private void updateChuanyunProjectWorkHour() {
        //@TODO
    }

    /**
     * 更新Manatee数据库每个项目工时数据
     */
    private void updateManateeProjectWorkHour() {
        //@TODO
    }

    /**
     * 删除多余每日工时
     *
     * @param chuanyunWorkHourList     日志原始工时
     * @param chuanyunSelfWorkHourList 标准化工时列表
     */
    private void compareDayWorkHour(List<ChuanyunWorkHourDO> chuanyunWorkHourList, List<ChuanyunSelfWorkHourDO> chuanyunSelfWorkHourList) {
        Map<String, Integer> map = new HashMap<>(64);
        //判断人员项目每日工时数量书否有变动
        if (chuanyunWorkHourList.size() != chuanyunSelfWorkHourList.size()) {
            //如果有变动将
            for (ChuanyunWorkHourDO chuanyunWorkHourDO : chuanyunWorkHourList) {
                map.put(chuanyunWorkHourDO.getUserId() + chuanyunWorkHourDO.getProjectId() +chuanyunWorkHourDO.getProjectType()+ chuanyunWorkHourDO.getDayLogDate(), 1);
            }
            //比较是否有撤回工时
            for (ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO : chuanyunSelfWorkHourList) {
//                if (!chuanyunSelfWorkHourDO.getProjectType().equals("生产项目")){
                    String projectId;
                    if (chuanyunSelfWorkHourDO.getProjectId().equals(ChuanYunConstant.PRODUCTION_PROJECT_ID)){
                        projectId="";
                    }else{
                        projectId=chuanyunSelfWorkHourDO.getProjectId();
                    }
                    Integer count = map.get(chuanyunSelfWorkHourDO.getUserId() + projectId +chuanyunSelfWorkHourDO.getProjectType()+chuanyunSelfWorkHourDO.getDayLogDate());
                    if (count == null) {
                        try {
                            //删除数据库日工时
                            chuanyunSelfWorkHourDao.delete(chuanyunSelfWorkHourDO);
                            //删除氚云日工时
                            List<String> matchers = new ArrayList<>();
                            if (!StringUtils.isEmpty(chuanyunSelfWorkHourDO.getProjectId())&&!chuanyunSelfWorkHourDO.getProjectId().equals(ChuanYunConstant.PRODUCTION_PROJECT_ID)){
                                matchers.add("F0000001_2," + chuanyunSelfWorkHourDO.getProjectId());
                            }
                            matchers.add("F0000002_2," + chuanyunSelfWorkHourDO.getUserId());
                            matchers.add("F0000003_2," + chuanyunSelfWorkHourDO.getDayLogDate());
                            var filter = Filter.instance(0, 1, true, "And", matchers);
                            var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMemberHourDO.SCHEMA_CODE, filter);
                            ChuanyunSaveDTO chuanyunSaveDTO;
                            if (chuanyunFindAllResponse.getReturnData() != null) {
                                List<ChuanyunMemberHourDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                                });
                                chuanyunSaveDTO = chuanYunManager.delete(ChuanyunMemberHourDO.SCHEMA_CODE, result.get(0).getObjectId());
                            }
                            log.info(chuanyunSelfWorkHourDO.getUserId() + "删除" + chuanyunSelfWorkHourDO.getDayLogDate() + "位于" + chuanyunSelfWorkHourDO.getProjectId() + "工时");
                        } catch (Exception e) {
                            e.printStackTrace();
                            log.error(e.getMessage());
                        }
                    }
//                }
            }
        }
    }

}
