package com.galaxis.manatee.service;

import com.galaxis.manatee.dao.ChuanyunSelfCostDao;
import com.galaxis.manatee.dao.ChuanyunSelfWorkHourDao;
import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.dao.ChuanyunWeekMemberHourDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSelfCostDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSelfWorkHourDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunWeekMemberHourDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSelfCostDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSelfWorkHourDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunWeekMemberHourDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.Instant;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2021/3/6 9:56 上午
 */
@Slf4j
@Service
public class LogUpdateService {

    private final DataArcherService dataArcherService;
    private final ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunWeekMemberHourDao chuanyunWeekMemberHourDao;
    private final ChuanyunSelfCostDao chuanyunSelfCostDao;

    public LogUpdateService(DataArcherService dataArcherService, ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunWeekMemberHourDao chuanyunWeekMemberHourDao, ChuanyunSelfCostDao chuanyunSelfCostDao) {
        this.dataArcherService = dataArcherService;
        this.chuanyunSelfWorkHourDao = chuanyunSelfWorkHourDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunWeekMemberHourDao = chuanyunWeekMemberHourDao;
        this.chuanyunSelfCostDao = chuanyunSelfCostDao;
    }

    /**
     * 更新最近一个月的标准每日工时
     */
    public void updateRecentChuanyunSelfWorkHour() {
        long startTime = Instant.now().getEpochSecond();
        var pageSize = 20;
        var page = 0;
        while (true) {
            var pageable = PageRequest.of(page, pageSize);
            var updateList = chuanyunSelfWorkHourDao.getRecentlyDayHour(pageable);
            if (page <= updateList.getTotalPages()) {
                page += 1;
                this.updateChuanyunSelfWorkHourList(updateList.toList());
            } else {
                break;
            }
        }
        log.info("更新最近一个月每日工时花费" + (Instant.now().getEpochSecond() - startTime) + "秒");
    }

    /**
     * 根据用户Id更新氚云每日工时列表
     *
     * @param userId 用户ID
     */
    public void updateChuanyunSelfWorkHourListByUserId(String userId) {
        List<ChuanyunSelfWorkHourDO> updateList = chuanyunSelfWorkHourDao.findByUserId(userId);
        this.updateChuanyunSelfWorkHourList(updateList);
    }

    /**
     * 根据用户Id更新氚云每周工时列表
     *
     * @param userId 用户ID
     */
    public void updateChuanyunWeekMemberHourListByUserId(String userId) {
        List<ChuanyunWeekMemberHourDO> updateList = chuanyunWeekMemberHourDao.findByUserId(userId);
        this.updateChuanyunWeekMemberHourList(updateList);
    }

    /**
     * 根据用户Id更新氚云每月工时列表
     *
     * @param userId 用户ID
     */
    public void updateChuanyunSelfCostListByUserId(String userId) {
        List<ChuanyunSelfCostDO> updateList = chuanyunSelfCostDao.findByUserId(userId);
        this.updateChuanyunSelfCostList(updateList);
    }

    /**
     * 根据标准日工时列表更新
     *
     * @param updateList 更新列表
     */
    public void updateChuanyunSelfWorkHourList(List<ChuanyunSelfWorkHourDO> updateList) {
        updateList.forEach(chuanyunSelfWorkHourDO -> {
            try {
                //
                ChuanyunSelfWorkHourDTO chuanyunSelfWorkHourDTO = new ChuanyunSelfWorkHourDTO();
                BeanUtils.copyProperties(chuanyunSelfWorkHourDO, chuanyunSelfWorkHourDTO);
                ChuanyunUserCompanyDO chuanyunUserCompanyDO = chuanyunUserCompanyDao.findByUserId(chuanyunSelfWorkHourDO.getUserId());
                if (chuanyunUserCompanyDO != null) {
                    chuanyunSelfWorkHourDTO.setUserName(chuanyunUserCompanyDO.getUserName());
                    chuanyunSelfWorkHourDTO.setBg(chuanyunUserCompanyDO.getBg());
                    //由于人员所在部门信息可能会表，所以每次更新的时候需要获取人员最新的部门信息
                    chuanyunSelfWorkHourDTO.setDepartmentId(chuanyunUserCompanyDO.getDepartmentId());
                } else {
                    chuanyunSelfWorkHourDTO.setUserName(chuanyunSelfWorkHourDTO.getUserId());
                    chuanyunSelfWorkHourDTO.setBg("IBG");
                    chuanyunSelfWorkHourDTO.setDepartmentId(chuanyunSelfWorkHourDO.getDepartmentId());
                }
                chuanyunSelfWorkHourDTO.setDepartmentName(chuanyunSelfWorkHourDO.getDepartmentName());
                //更新到氚云
                dataArcherService.updateSelfWorkHour(chuanyunSelfWorkHourDTO);

            } catch (Exception e) {
                log.error("更新标准工时异常", e);
            }
        });
    }

    /**
     * 根据标准周工时列表更新
     *
     * @param updateList 更新列表
     */
    public void updateChuanyunWeekMemberHourList(List<ChuanyunWeekMemberHourDO> updateList) {
        updateList.forEach(chuanyunWeekMemberHourDO -> {
            //更新氚云
            try {
                String departmentId = chuanyunUserCompanyDao.findByUserId(chuanyunWeekMemberHourDO.getUserId()).getDepartmentId();
                ChuanyunWeekMemberHourDTO chuanyunWeekMemberHourDTO = new ChuanyunWeekMemberHourDTO();
                chuanyunWeekMemberHourDTO.setProjectId(chuanyunWeekMemberHourDO.getProjectId());
                chuanyunWeekMemberHourDTO.setUserId(chuanyunWeekMemberHourDO.getUserId());
                chuanyunWeekMemberHourDTO.setWeek(chuanyunWeekMemberHourDO.getWeek());
                chuanyunWeekMemberHourDTO.setProjectType(chuanyunWeekMemberHourDO.getProjectType());
                chuanyunWeekMemberHourDTO.setTotalHour(chuanyunWeekMemberHourDO.getStandardWorkHour());
                chuanyunWeekMemberHourDTO.setStatus(chuanyunWeekMemberHourDO.getStatus());
                chuanyunWeekMemberHourDTO.setYear(chuanyunWeekMemberHourDO.getYear());
                chuanyunWeekMemberHourDTO.setDepartmentId(departmentId);
                dataArcherService.updateWeekMemberHour(chuanyunWeekMemberHourDTO);
            } catch (Exception e) {
                e.printStackTrace();
                log.warn("更新每周工时异常" + chuanyunWeekMemberHourDO);
            }
        });
    }

    /**
     * 根据标准月工时列表更新
     *
     * @param updateList 更新列表
     */
    public void updateChuanyunSelfCostList(List<ChuanyunSelfCostDO> updateList) {
        updateList.forEach(chuanyunSelfCostDO -> {
            try {
                ChuanyunSelfCostDTO chuanyunSelfCostDTO = new ChuanyunSelfCostDTO();
                //更新氚云数据
                chuanyunSelfCostDTO.setCost(chuanyunSelfCostDO.getCost());
                chuanyunSelfCostDTO.setProjectId(chuanyunSelfCostDO.getProjectId());
                chuanyunSelfCostDTO.setProjectType(chuanyunSelfCostDO.getProjectType());
                chuanyunSelfCostDTO.setStatus(chuanyunSelfCostDO.getStatus());
                chuanyunSelfCostDTO.setUserId(chuanyunSelfCostDO.getUserId());
                chuanyunSelfCostDTO.setMonthWorkHour(chuanyunSelfCostDO.getMonthlyTotalHour());
                chuanyunSelfCostDTO.setDayLogDate(chuanyunSelfCostDO.getDayLogDate());
                chuanyunSelfCostDTO.setWorkHour(chuanyunSelfCostDO.getTotalHour());
                chuanyunSelfCostDTO.setDepartmentId(chuanyunSelfCostDO.getDepartmentId());
                if (!StringUtils.isEmpty(chuanyunSelfCostDO.getProjectId())) {
                    dataArcherService.updateChuanyunSelfCost(chuanyunSelfCostDTO);
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.warn("更新每月工时异常" + chuanyunSelfCostDO);
            }
        });
    }
}
