package com.galaxis.manatee.service.impl;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.constant.StringConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.service.ChuanyunService;
import com.galaxis.manatee.task.ChuanyunBasicDataScheduledTask;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.CascadeType;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author chongjiezhong
 */
@Slf4j
@Service
public class ChuanyunServiceImpl implements ChuanyunService {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunCostCollectionDao chuanyunCostCollectionDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunEquipmentInformDao chuanyunEquipmentInformDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask;
    private final ChuanyunOperationInformationDao chuanyunOperationInformationDao;

    public ChuanyunServiceImpl(ChuanYunManager chuanYunManager, ChuanyunCostCollectionDao chuanyunCostCollectionDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunEquipmentInformDao chuanyunEquipmentInformDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask, ChuanyunOperationInformationDao chuanyunOperationInformationDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunCostCollectionDao = chuanyunCostCollectionDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunEquipmentInformDao = chuanyunEquipmentInformDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunBasicDataScheduledTask = chuanyunBasicDataScheduledTask;
        this.chuanyunOperationInformationDao = chuanyunOperationInformationDao;
    }

    @Override
    public void updateProjectMember() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {

                    //处理项目对象中的项目成员数据(改为手动调用处理)
                    projectMemberCheck(chuanyunGroupProjectDO);
                    //只处理项目成员为空的情况
                    boolean emptyFlag= chuanyunGroupProjectDO.getProjectMemberList()==null||chuanyunGroupProjectDO.getProjectMemberList().length==0;
                    if(emptyFlag){
                        //增加项目成员
                        addMemberList(chuanyunGroupProjectDO);
                        GroupProjectUpdateDTO groupProjectUpdateDTO= new GroupProjectUpdateDTO(chuanyunGroupProjectDO);
                        //生成更新数据后校验数据是否增加成功
                        if(groupProjectUpdateDTO.getProjectMemberList()!=null&&groupProjectUpdateDTO.getProjectMemberList().length!=0){
                            try {
                                var groupProjectUpdateString=objectMapper.writeValueAsString(groupProjectUpdateDTO);
                                ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
                                if(chuanyunSaveDTO.getSuccessful()){
                                    log.info(chuanyunGroupProjectDO.getObjectId()+"更新项目组成员成功");

                                }else{
                                    log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目组成员失败");
                                    log.warn(chuanyunGroupProjectDO+"");
                                }
                            } catch (BigSizeException | JsonProcessingException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                });
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
    }

    @Override
    public List<ChuanyunCostCollectionDO> getAllCostCollection() {
        return chuanyunCostCollectionDao.findAll();
    }

    @Override
    public void updateChuanyunDept(String code) {
        var objectMapper=new ObjectMapper();
        var start=0;
        var flag=true;
        List<String> matchersString=new ArrayList<>();
        matchersString.add("Status_2,1");
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true,Filter.AND,matchersString);
                var chuanyunFindAllResponse=chuanYunManager.findAll(code,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunUpdateDeptDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunUpdateDeptDO ->  {
                    //更新拥有者id，更新所属部门
                    chuanyunUpdateDeptDO.setOwnerId(chuanyunUpdateDeptDO.getOwnerIdObject().getObjectId());
                    ChuanyunUserCompanyDO chuanyunUserCompanyDO = chuanyunUserCompanyDao.findByUserId(chuanyunUpdateDeptDO.getOwnerIdObject().getObjectId());
                    chuanyunUpdateDeptDO.setOwnerDepartmentId(chuanyunUserCompanyDO.getDepartmentId());
                    ProjectUpdateDTO projectUpdateDTO = new ProjectUpdateDTO(chuanyunUpdateDeptDO);
                    try {
                        var groupProjectUpdateString=objectMapper.writeValueAsString(projectUpdateDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(code,chuanyunUpdateDeptDO.getObjectId(),groupProjectUpdateString);
                        if(chuanyunSaveDTO.getSuccessful()){
                            log.info(chuanyunUpdateDeptDO.getObjectId()+"更新项目拥有者成功");
                        }else{
                            log.warn(chuanyunUpdateDeptDO.getObjectId()+"更新项目拥有者失败");
                            log.warn(chuanyunUpdateDeptDO+"");
                        }
                    } catch (JsonProcessingException | BigSizeException e) {
                        e.printStackTrace();
                    }

                });
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
    }


    @Override
    public void importEquipment(List<ChuanyunEquipmentDO> list) {
        var objectMapper=new ObjectMapper();
        list.forEach(chuanyunEquipmentDO -> {
            //更新氚云
            try {
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000001_2,"+chuanyunEquipmentDO.getEquipmentCode());
                var filter= Filter.instance(0,1,true,"And",matchers);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunEquipmentInformDO.SCHEMA_CODE,filter);
                if (chuanyunFindAllResponse.getReturnData()!=null){
                    List<ChuanyunEquipmentInformDO> projectresult=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                    chuanyunEquipmentDO.setEquipmentCode(projectresult.get(0).getObjectId());
                }
//                ChuanyunEquipmentInformDO chuanyunEquipmentInformDO = chuanyunEquipmentInformDao.findByEquipmentCode(chuanyunEquipmentDO.getEquipmentCode());
//                if (chuanyunEquipmentInformDO!=null){
//                    chuanyunEquipmentDO.setEquipmentCode(chuanyunEquipmentInformDO.getObjectId());
//                }
                EquipmentDTO equipmentDTO = new EquipmentDTO(chuanyunEquipmentDO);
                var equipmentString = objectMapper.writeValueAsString(equipmentDTO);
                ChuanyunSaveDTO chuanyunSaveDTO;
//                if (chuanyunFindAllResponse.getReturnData()!=null) {
//                    List<ChuanyunEquipmentDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
//                    chuanyunSaveDTO = chuanYunManager.update(ChuanyunEquipmentDO.SCHEMA_CODE,result.get(0).getObjectId(),equipmentString);
//                }else {
                    chuanyunSaveDTO = chuanYunManager.save(ChuanyunEquipmentDO.SCHEMA_CODE,equipmentString,true);
//                }
                if (chuanyunSaveDTO.getSuccessful()){
                    log.info("新增项目配置信息成功");
                }else {
                    log.error("新增项目配置信息失败");
                }
            } catch (JsonProcessingException | BigSizeException e) {
                e.printStackTrace();
            }
        });
    }

    @Override
    public void updateU9Department() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try{
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunU9DepartmentDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                List<ChuanyunU9DepartmentDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunU9DepartmentDO -> {
                    synchroOutDepartment(chuanyunU9DepartmentDO);
                });
            }catch (JsonProcessingException  e){
                e.printStackTrace();
                log.warn(e.getMessage());
            }
        }
    }

    @Override
    public void updateOperationInformation(ChuanyunOperationInformationDO chuanyunOperationInformationDO) {
        chuanyunOperationInformationDO.setCreatetime(LocalDateTime.now());
        chuanyunOperationInformationDao.save(chuanyunOperationInformationDO);
    }

    @Override
    public void importProduct(List<ChuanyunProductDO> list) {
        try {
            ChuanyunProductLibraryDO chuanyunProductLibraryDO = new ChuanyunProductLibraryDO();
            List<ChuanyunProductLibrarySpecificationsDO> list1 = new ArrayList<>();
            list.forEach(chuanyunProductDO -> {
                if (!StringUtils.isEmpty(chuanyunProductDO.getProductModel())){
                    chuanyunProductLibraryDO.setProductModel(chuanyunProductDO.getProductModel());
                }
                ChuanyunProductLibrarySpecificationsDO chuanyunProductLibrarySpecificationsDO = new ChuanyunProductLibrarySpecificationsDO();
                chuanyunProductLibrarySpecificationsDO.setSpecName(chuanyunProductDO.getSpecName());
                chuanyunProductLibrarySpecificationsDO.setSpecValue(chuanyunProductDO.getSpecValue());
                chuanyunProductLibrarySpecificationsDO.setRemark(chuanyunProductDO.getRemark());
                list1.add(chuanyunProductLibrarySpecificationsDO);
            });
            var objectMapper=new ObjectMapper();
            List<String> matchers = new ArrayList<>();
            matchers.add("F0000004_2,"+chuanyunProductLibraryDO.getProductModel());
            var filter= Filter.instance(0,1,true,"And",matchers);
            var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProductLibraryDO.SCHEMA_CODE,filter);
            if (chuanyunFindAllResponse.getReturnData()!=null) {
                List<ChuanyunProductLibraryDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                chuanYunManager.delete(ChuanyunProductLibraryDO.SCHEMA_CODE,result.get(0).getObjectId());
            }
            ProductLibraryDTO productLibraryDTO = new ProductLibraryDTO(chuanyunProductLibraryDO);
            productLibraryDTO.setList(list1);
            var productString = objectMapper.writeValueAsString(productLibraryDTO);
            ChuanyunSaveDTO chuanyunSaveDTO = chuanYunManager.save(ChuanyunProductLibraryDO.SCHEMA_CODE,productString,true);
            if(chuanyunSaveDTO.getSuccessful()){
                log.info("更新产品库成功");
            }else{
                log.warn("更新产品库失败");
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error(e.getMessage());
        }
    }

    /**
     * 更新外部部门名称
     * @param
     */
    private void synchroOutDepartment(ChuanyunU9DepartmentDO chuanyunU9DepartmentDO) {
        try {
            if (!StringUtils.isEmpty(chuanyunU9DepartmentDO.getActualDepartmentObject().getObjectId())){
                var objectMapper=new ObjectMapper();
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000001_2,"+chuanyunU9DepartmentDO.getActualDepartmentObject().getObjectId());
                var filter= Filter.instance(0,1,true,"And",matchers);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunOutDepartmentDO.SCHEMA_CODE,filter);
                if (chuanyunFindAllResponse.getReturnData()!=null) {
                    List<ChuanyunOutDepartmentDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                    ChuanyunOutDepartmentDO firstResult = result.get(0);
                    chuanyunU9DepartmentDO.setDepartmentName(firstResult.getOutDepartment());
                    chuanyunU9DepartmentDO.setDepartmentCode(firstResult.getDepartmentCode());
                }
                outDepartmentDTO departmentDTO = new outDepartmentDTO(chuanyunU9DepartmentDO);
                var departmentString = objectMapper.writeValueAsString(departmentDTO);
                ChuanyunSaveDTO chuanyunSaveDTO = chuanYunManager.update(ChuanyunU9DepartmentDO.SCHEMA_CODE,chuanyunU9DepartmentDO.getObjectId(),departmentString);
                if(chuanyunSaveDTO.getSuccessful()){
                    log.info("更新外部部门名称成功");
                }else{
                    log.warn("更新外部部门名称失败");
                }
            }
        }catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }

    }

    /**
     * 如果没有项目组成员，判断是否是EBG的实施、研发二开、售后维保项目，如果是，则添加EBG全体成员
     * @param chuanyunGroupProjectDO    项目对象
     */
    private void addMemberList(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        //项目成员为空的情况判断
        //判断项目BG是否为EBG
        var specifiedBusinessGroupFlag=ChuanyunGroupProjectDO.EBG.equals(chuanyunGroupProjectDO.getBusinessGroup());
        //判断项目类型是否为 下列项目类型中的一种
        var specifiedProjectTypeFlag=("实施".equals(chuanyunGroupProjectDO.getProjectType())||
                "维保".equals(chuanyunGroupProjectDO.getProjectType())||
                "研发二开".equals(chuanyunGroupProjectDO.getProjectType())||
                "售后".equals(chuanyunGroupProjectDO.getProjectType()));
        //如果项目组成员为空，则自动为项目添加所有EBG成员作为项目组成员
        if (specifiedBusinessGroupFlag&&specifiedProjectTypeFlag) {
            var projectMember = new ArrayList<String>();
            var builder = new StringBuilder();
            ChuanYunConstant.USER_BUSINESS_GROUP.forEach((k, v)->{
                if (ChuanyunGroupProjectDO.EBG.equals(v)&&null!=k){
                    builder.append(k).append(StringConstant.SEMICOLON);
                    projectMember.add(k);
                }
            });

            chuanyunGroupProjectDO.setProjectMember(builder.toString());
            chuanyunGroupProjectDO.setProjectMemberList(projectMember.toArray(new String[0]));
            log.info("成员已更新"+chuanyunGroupProjectDO.getProjectCode()+chuanyunGroupProjectDO.getProjectType());
        }
    }

    /**
     * 如果有项目组成员，将项目组成员projectMember替换为主键
     * @param chuanyunGroupProjectDO    项目对象
     */
    private void projectMemberCheck(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        if (chuanyunGroupProjectDO.getProjectMemberList() != null && chuanyunGroupProjectDO.getProjectMemberList().length>0) {
            List<String> memberList = new ArrayList<>();
            var builder = new StringBuilder();
            for (ChuanyunObject chuanyunObject: chuanyunGroupProjectDO.getProjectMemberObject()) {
                builder.append(chuanyunObject.getObjectId()).append(StringConstant.SEMICOLON);
                memberList.add(chuanyunObject.getObjectId());
            }
            chuanyunGroupProjectDO.setProjectMember(builder.toString());
            //替换为主键后更新氚云数据表来增加权限
            chuanyunGroupProjectDO.setProjectMemberList(memberList.toArray(new String[0]));
        }
    }

    /**
     * 用于刷新项目经理权限的对象
     */
    @Data
    static class GroupProjectUpdateDTO{

        /**
         * 项目组成员（前端传数据用）
         */
        @JsonProperty("F0000034")
        private String[] projectMemberList;

        public GroupProjectUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectMemberList()){
                    this.projectMemberList =chuanyunGroupProjectDO.getProjectMemberList();
                }

            }
        }
    }

    @Data
    static class ProductLibraryDTO{

        @JsonProperty("F0000004")
        private String productModel;

        @JsonProperty("D001789F4d732f3541934286b97c21a1ea1e5398")
        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true,fetch = FetchType.EAGER)
        private List<ChuanyunProductLibrarySpecificationsDO> list;

        public ProductLibraryDTO(ChuanyunProductLibraryDO chuanyunProductLibraryDO) {
            if(null!=chuanyunProductLibraryDO){
                if (null!=chuanyunProductLibraryDO.getProductModel()){
                    this.productModel=chuanyunProductLibraryDO.getProductModel();
                }
            }
        }
    }


    @Data
    class outDepartmentDTO {

        @JsonProperty("F0000062")
        private String outDepartment;

        @JsonProperty("F0000063")
        private String departmentCode;

        public outDepartmentDTO(ChuanyunU9DepartmentDO chuanyunU9DepartmentDO) {
            if (null!=chuanyunU9DepartmentDO){
                if(null!=chuanyunU9DepartmentDO.getDepartmentName()){
                    this.outDepartment=chuanyunU9DepartmentDO.getDepartmentName();
                }
                if (null!=chuanyunU9DepartmentDO.getDepartmentCode()){
                    this.departmentCode=chuanyunU9DepartmentDO.getDepartmentCode();
                }
            }
        }
    }

    @Data
    class EquipmentDTO{
        /**
         * 设备编号
         */
        @JsonProperty("F0000001")
        private String equipmentCode;

        /**
         * 设备配置编号
         */
        @JsonProperty("F0000002")
        private String equipmentConfigureCode;

        /**
         * 设备配置属性
         */
        @JsonProperty("F0000003")
        private String equipmentConfigureAttribute;

        /**
         * 设备配置值
         */
        @JsonProperty("F0000004")
        private String equipmentConfigureValue;

        public EquipmentDTO(ChuanyunEquipmentDO chuanyunEquipmentDO){
            if (null!=chuanyunEquipmentDO){
                if (null!=chuanyunEquipmentDO.getEquipmentCode()){
                    this.equipmentCode=chuanyunEquipmentDO.getEquipmentCode();
                }
                if (null!=chuanyunEquipmentDO.getEquipmentConfigureAttribute()){
                    this.equipmentConfigureAttribute=chuanyunEquipmentDO.getEquipmentConfigureAttribute();
                }
                if (null!=chuanyunEquipmentDO.getEquipmentConfigureCode()){
                    this.equipmentConfigureCode=chuanyunEquipmentDO.getEquipmentConfigureCode();
                }
                if (null!=chuanyunEquipmentDO.getEquipmentConfigureValue()){
                    this.equipmentConfigureValue=chuanyunEquipmentDO.getEquipmentConfigureValue();
                }
            }
        }
    }

    @Data
    static class ProjectUpdateDTO{

        @JsonProperty("OwnerId")
        private String ownerId;

        /**
         * 拥有者所在部门Id
         */
        @JsonProperty("OwnerDeptId")
        private String ownerDepartmentId;

        public ProjectUpdateDTO(ChuanyunUpdateDeptDO chuanyunUpdateDeptDO) {
            if(null!=chuanyunUpdateDeptDO) {
                if(null!=chuanyunUpdateDeptDO.getOwnerDepartmentId()) {
                    this.ownerDepartmentId = chuanyunUpdateDeptDO.getOwnerDepartmentId();
                }
            }
        }
    }


}
