package com.galaxis.manatee.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/22 11:33 下午
 */
@Slf4j
@Service
public class ChuanyunHumanResourceScheduledTask {

    private final Integer pageSize = 100;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunLogDao chuanyunLogDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunHolidaySalaryRuleDao chuanyunHolidaySalaryRuleDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunPayrollDao chuanyunPayrollDao;
    private final ChuanyunUserDepartmentNameDao chuanyunUserDepartmentNameDao;

    public ChuanyunHumanResourceScheduledTask(ChuanYunManager chuanYunManager, ChuanyunLogDao chuanyunLogDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunHolidaySalaryRuleDao chuanyunHolidaySalaryRuleDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunPayrollDao chuanyunPayrollDao, ChuanyunUserDepartmentNameDao chuanyunUserDepartmentNameDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunLogDao = chuanyunLogDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunHolidaySalaryRuleDao = chuanyunHolidaySalaryRuleDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunPayrollDao = chuanyunPayrollDao;
        this.chuanyunUserDepartmentNameDao = chuanyunUserDepartmentNameDao;
    }

    /**
     * 提供一个外部调用接口
     */
    public void getPayrollManually(){
        this.getPayroll();
    }

    public void getUpdateLog() {
        this.updateLog();
    }

    /**
     * 每天3点更新日志记录
     */
    @Scheduled(cron = "0 0 1 * * *")
    private void updateLog() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        List<String> matchersString=new ArrayList<>();
        //由于需要统计流程中的日志，在日志数据同步时将状态为2的数据一并统计
//        matchersString.add("Status_2,1");
        matchersString.add("Status_6,[1;2;3]");
//        matchersString.add("logDate_4,2019-01-01 00:00:00");
        while (flag) {
            Long startTime= Instant.now().getEpochSecond();
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true,Filter.AND,matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                result.forEach(chuanyunLogDO -> {
                    if(chuanyunLogDO.getDayLogList()==null||chuanyunLogDO.getDayLogList().size()==0){
                        chuanyunLogDO.setDayLogList(new ArrayList<>());
                        log.warn(chuanyunLogDO+"");
                        log.warn("异常数据");
                    }else{
                        //如果有重复就删除
                        chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(toDelete -> {
//                            log.info("删除" + toDelete.getObjectId());
                            chuanyunLogDao.delete(toDelete);
                        });
                        //根据projectId获取code,如果是生产项目，则不用处理
                        //针对前期测试数据，可能出现日志为空的情况，跳过不处理
                        chuanyunLogDO.getDayLogList().forEach(chuanyunDayLog -> {
                            try{
                                //离职人员补全用户Id及部门ID
                                if (StringUtils.isEmpty(chuanyunLogDO.getUserId())){
                                    var name="";
                                    if (StringUtils.isEmpty(chuanyunDayLog.getContent())){
                                        name=chuanyunLogDO.getName();
                                    }else {
                                        name=chuanyunDayLog.getContent();
                                    }
                                    chuanyunLogDO.setUserId(getCharacters(name));
                                    ChuanyunUserDepartmentNameDO chuanyunUserDepartmentNameDO = chuanyunUserDepartmentNameDao.findByUserName(chuanyunLogDO.getUserId());
                                    if (chuanyunUserDepartmentNameDO!=null){
                                        chuanyunLogDO.setDepartmentName(chuanyunUserDepartmentNameDO.getDepartmentName());
                                    }
                                }else {
                                  ChuanyunUserCompanyDO chuanyunUserCompanyDO = chuanyunUserCompanyDao.findByUserId(chuanyunLogDO.getUserObject().getObjectId());
                                  if (chuanyunUserCompanyDO!=null){
                                      chuanyunLogDO.setDepartmentName(getDepartmentName(chuanyunUserCompanyDO.getDepartment()));
                                  }
                                }

                                if(!"生产项目".equals(chuanyunDayLog.getProjectType())&& StringUtils.isEmpty(chuanyunDayLog.getProjectId())){
                                    log.info(chuanyunLogDO+"");
                                }else if(!"生产项目".equals(chuanyunDayLog.getProjectType())){
                                    ChuanyunGroupProjectDO chuanyunGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunDayLog.getProjectId()).orElseThrow();
                                    chuanyunDayLog.setProjectCode(chuanyunGroupProjectDO.getProjectCode());
                                }
                            }catch (Exception e){
                                e.printStackTrace();
                                log.warn(chuanyunLogDO+"");
                            }
                        });
                        //判断所属部门是否正确
                        //目前已经通过excel方式更新完成这个方法暂时不需要使用
//                        checkDepartment(chuanyunLogDO);
                    }
                    //处理项目userId
                    if(chuanyunLogDO.getUserObject()!=null){
                        chuanyunLogDO.setUserId(chuanyunLogDO.getUserObject().getObjectId());
                    }
                });
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAll(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info(Instant.now().getEpochSecond()-startTime+"日志100条保存时间");
        }
        log.info("历史日志信息保存" + totalCount);
    }

    /**
     * 检查日志中人员所属部门和日志所属部门是否一致，用于处理手工excel导入日志造成的日志部门不一致的情况
     * @param chuanyunLogDO 日志对象
     */
    @Deprecated
    private void checkDepartment(ChuanyunLogDO chuanyunLogDO){
        var objectMapper=new ObjectMapper();
        chuanyunUserCompanyDao.findById(chuanyunLogDO.getUserCompanyId()).ifPresent(chuanyunUserCompanyDO -> {
            String ownerDepartmentId = chuanyunUserCompanyDO.getDepartment();
            if (!chuanyunLogDO.getOwnerDepartmentId().equals(ownerDepartmentId)) {
                chuanyunLogDO.setOwnerDepartmentId(ChuanYunConstant.USER_DEPARTMENT.get(chuanyunUserCompanyDO.getUserId()).toString());
                boolean updateFlag=false;
                while (!updateFlag) {
                    try {
                        var groupProjectUpdateString = objectMapper.writeValueAsString(chuanyunLogDO);
                        ChuanyunSaveDTO chuanyunSaveDTO = chuanYunManager.update(ChuanyunLogDO.SCHEMA_CODE, chuanyunLogDO.getObjectId(), groupProjectUpdateString);
                        if (chuanyunSaveDTO.getSuccessful()) {
                            updateFlag = true;
                        } else {
                            log.warn(chuanyunLogDO.getObjectId() + "更新人员所属部门失败" + groupProjectUpdateString);
                            Thread.sleep(5000L);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        });
    }

    /**
     * 法定假日薪资规则同步
     */
    @Scheduled(fixedDelay = 86400000L)
    private void getHolidaySalaryRule() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunHolidaySalaryRuleDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunHolidaySalaryRuleDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存数据
                chuanyunHolidaySalaryRuleDao.saveAll(result);
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("法定假日薪资规则同步"+totalCount);
    }

    /**
     * 凯乐士薪酬项同步
     */
    @Scheduled(fixedDelay = 86400000L)
    private void getPayroll() {
        chuanyunPayrollDao.deleteAll();
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPayrollDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunPayrollDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存数据
                chuanyunPayrollDao.saveAll(result);
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("凯乐士薪酬项同步"+totalCount);
    }

//    @Scheduled(fixedDelay = 3600000L)
    private void updateRecentLog() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        List<String> matchersString=new ArrayList<>();
        //由于需要统计流程中的日志，在日志数据同步时将状态为2的数据一并统计
        matchersString.add("Status_6,[1;2]");
//        matchersString.add("logDate_1,2020-08-01 00:00:00");
        while (flag) {
            Long startTime= Instant.now().getEpochSecond();
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true,Filter.AND,matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                result.forEach(chuanyunLogDO -> {
                    if(chuanyunLogDO.getDayLogList()==null||chuanyunLogDO.getDayLogList().size()==0){
                        chuanyunLogDO.setDayLogList(new ArrayList<>());
                        log.warn(chuanyunLogDO+"");
                        log.warn("异常数据");
                    }else{
                        //如果有重复就删除
                        chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(toDelete -> {
//                            log.info("删除" + toDelete.getObjectId());
                            chuanyunLogDao.delete(toDelete);
                        });
                        //根据projectId获取code,如果是生产项目，则不用处理
                        //针对前期测试数据，可能出现日志为空的情况，跳过不处理
                        chuanyunLogDO.getDayLogList().forEach(chuanyunDayLog -> {
                            try{
                                if(!"生产项目".equals(chuanyunDayLog.getProjectType())&& StringUtils.isEmpty(chuanyunDayLog.getProjectId())){
                                    log.info(chuanyunLogDO+"");
                                }else if(!"生产项目".equals(chuanyunDayLog.getProjectType())){
                                    ChuanyunGroupProjectDO chuanyunGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunDayLog.getProjectId()).orElseThrow();
                                    chuanyunDayLog.setProjectCode(chuanyunGroupProjectDO.getProjectCode());
                                }
                            }catch (Exception e){
                                e.printStackTrace();
                                log.warn(chuanyunDayLog+"");
                            }
                        });
                        //判断所属部门是否正确
                        //目前已经通过excel方式更新完成这个方法暂时不需要使用
//                        checkDepartment(chuanyunLogDO);
                    }
                    //处理项目userId
                    if(chuanyunLogDO.getUserObject()!=null){
                        chuanyunLogDO.setUserId(chuanyunLogDO.getUserObject().getObjectId());
                    }
                });
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAll(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info(Instant.now().getEpochSecond()-startTime+"日志100条保存时间");
        }
        log.info("最近的日志日志信息保存" + totalCount);
    }

    private String getCharacters(String str){
        String reg = "[^\u4e00-\u9fa5]";
        str = str.replaceAll(reg, " ");
        return str.trim();
    }

    private String getDepartmentName(String str){
        if (str.contains("/")){
            String[] department = str.split("/");
            return department[department.length-1];
        }else {
            return str;
        }
    }
}
