package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/1 11:37 上午
 */
@Slf4j
@Service
public class ChuanyunProjectScheduledTask {

    private final Integer pageSize=20; final ObjectMapper objectMapper=new ObjectMapper();
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunSaleContractDao chuanyunSaleContractDao;
    private final ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao;
    private final ChuanyunPurCostDao chuanyunPurCostDao;
    private final ChuanyunProjectCashFlowDao chuanyunProjectCashFlowDao;
    private final ChuanyunMonthLaborCostDao chuanyunMonthLaborCostDao;

    public ChuanyunProjectScheduledTask(ChuanYunManager chuanYunManager, ChuanyunSaleContractDao chuanyunSaleContractDao, ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao, ChuanyunPurCostDao chuanyunPurCostDao, ChuanyunProjectCashFlowDao chuanyunProjectCashFlowDao, ChuanyunMonthLaborCostDao chuanyunMonthLaborCostDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunSaleContractDao = chuanyunSaleContractDao;
        this.chuanyunPaymentReceiveDao = chuanyunPaymentReceiveDao;
        this.chuanyunPurCostDao = chuanyunPurCostDao;
        this.chuanyunProjectCashFlowDao = chuanyunProjectCashFlowDao;
        this.chuanyunMonthLaborCostDao = chuanyunMonthLaborCostDao;
    }

    /**
     * 更新销售合同数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSaleContractDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSaleContractDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSaleContractDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同信息保存保存"+totalCount);
    }

    /**
     * 同步项目人力成本
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getLaborCost() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMonthLaborCostDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunMonthLaborCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunMonthLaborCostDO -> {
                    var tmp=chuanyunMonthLaborCostDao.findById(chuanyunMonthLaborCostDO.getObjectId());
                    tmp.ifPresent(chuanyunMonthLaborCostDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunMonthLaborCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目人力成本保存"+totalCount);
    }

    /**
     * 同步项目收付款
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getCashFlow() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectCashFlowDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunProjectCashFlowDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunProjectCashFlowDO -> {
                    var tmp=chuanyunProjectCashFlowDao.findById(chuanyunProjectCashFlowDO.getObjectId());
                    tmp.ifPresent(chuanyunProjectCashFlowDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunProjectCashFlowDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目收付款保存"+totalCount);
    }

    /**
     * 同步项目物料成本
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getPurCost() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPurCostDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunPurCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPurCostDO -> {
                    var tmp=chuanyunPurCostDao.findById(chuanyunPurCostDO.getObjectId());
                    tmp.ifPresent(chuanyunPurCostDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPurCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
            }
        log.info("项目物料成本保存"+totalCount);
    }
    /**
     * 更新销售合同回款数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractPaymentReceiveFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunPaymentReceiveDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunPaymentReceiveDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPaymentReceiveDO -> {
                    var tmp=chuanyunPaymentReceiveDao.findById(chuanyunPaymentReceiveDO.getObjectId());
                    tmp.ifPresent(chuanyunPaymentReceiveDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPaymentReceiveDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同回款信息保存保存"+totalCount);
    }







    /**
     * 项目汇总工时
     */
    @Data
    static
    class ProjectTotalCostDTO{

        /**
         * 项目ID
         */
        @JsonProperty("F0000001")
        private String projectId;

        /**
         * 项目类型
         */
        @JsonProperty("F0000004")
        private String projectType;

        @JsonProperty("F0000008")
        private String projectCode;

        /**
         * 工时
         */
        @JsonProperty("F0000003")
        private BigDecimal workHour;

        /**
         * 总人天
         */
        @JsonProperty("F0000005")
        private BigDecimal personDays;

        /**
         * 工时状态
         */
        @JsonProperty("F0000006")
        private String status;

    }

    @Data
    static
    class ChuanyunAdjustAccountDTO {
        @JsonProperty("F0000002")
        private String projectCode;

        /**
         * 核算实施人天
         */
        @JsonProperty("F0000003")
        private BigDecimal personDays;

        /**
         * 核算实施人力成本
         */
        @JsonProperty("F0000004")
        private BigDecimal laborCost;

        /**
         * 核算实施费用
         */
        @JsonProperty("F0000005")
        private BigDecimal implementationCosts;

        /**
         *核算实施总成本
         */
        @JsonProperty("F0000006")
        private BigDecimal totalImplementationCosts;

        /**
         * 项目
         */
        @JsonProperty("F0000007")
        private String projectId;
    }
}

