package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.constant.DingTalkConstant;
import com.galaxis.manatee.constant.StringConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.entity.chuanyun.bo.MaterialResendBO;
import com.galaxis.manatee.entity.ding.DingTalkProcessInstance;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/2/13 11:53 上午
 */
@Service
@Slf4j
public class ChuanyunScheduledTask {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunProjectDao chuanyunProjectDao;
    private final ChuanyunMaterialDao chuanyunMaterialDao;
    private final DingTalkProcessInstanceDao dingTalkProcessInstanceDao;
    private final ClawFeign clawFeign;

    public ChuanyunScheduledTask(ChuanYunManager chuanYunManager, ChuanyunProjectDao chuanyunProjectDao, ChuanyunMaterialDao chuanyunMaterialDao, DingTalkProcessInstanceDao dingTalkProcessInstanceDao, ClawFeign clawFeign) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunProjectDao = chuanyunProjectDao;
        this.chuanyunMaterialDao = chuanyunMaterialDao;
        this.dingTalkProcessInstanceDao = dingTalkProcessInstanceDao;
        this.clawFeign = clawFeign;
    }

    /**
     * 定时将氚云中的数据保存到manatee
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getDataFromChuanyun(){
        //获取项目数据
//        materialInformationSync();
        getProjectInformationFromChuanyun();
    }

    /**
     * 定时将物料信息从氚云中同步
     */
    public void materialInformationSync(){
        var pageSize=100;
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter=Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunMaterialDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                var objectMapper=new ObjectMapper();
                List<ChuanyunMaterialDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //保存
                result.removeIf(chuanyunMaterialDO -> chuanyunMaterialDao.findById(chuanyunMaterialDO.getObjectId()).isPresent());
                chuanyunMaterialDao.saveAll(result);
                log.info("保存物料信息"+result.size());
                totalCount+=result.size();
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("物料信息保存保存"+totalCount);
        //将氚云数据同步到mysql之后将U9的数据同步到氚云
        getMaterialFromU9();
    }

    /**
     * 将U9中物料信息更新到氚云
     */
    private void getMaterialFromU9(){
        int page=1;
        int size=50;
        var totalAmount=0;
        var totalPage=Integer.MAX_VALUE;
        Map<String,String> parameters=new HashMap<>(2);
        parameters.put("page", Integer.toString(page));
        parameters.put("size", Integer.toString(size));

        while (page<=totalPage){
            List<ChuanyunMaterialDO> toSave=new ArrayList<>();
            //获取U9中的物料数据
            Page<MaterialDTO> result=clawFeign.findMaterialPageable(parameters);
            totalPage=result.getTotalPages();
            List<ChuanyunMaterialDO> chuanyunMaterialList=new ArrayList<>();
            //遍历获取的数据，如果系统中已经存在就不进行保存
            result.getContent().forEach(materialDTO -> {
                List<ChuanyunMaterialDO> list=chuanyunMaterialDao.findAllByCode(materialDTO.getMaterialPrimaryKey().getCode());
                if(list.size()==0){
                    ChuanyunMaterialDO chuanyunMaterialDO=ChuanyunMaterialDO.fromMaterialDTO(materialDTO);
                    chuanyunMaterialList.add(chuanyunMaterialDO);
                    toSave.add(chuanyunMaterialDO);
                }
            });
            chuanyunMaterialDao.saveAll(chuanyunMaterialList);
            totalAmount+=chuanyunMaterialList.size();
            page++;
            parameters.put("page",Integer.toString(page));
            log.info("共"+totalPage+"页，当前第"+page+"页，更新了"+chuanyunMaterialList.size()+"条数据");
            //将更新的的物料信息同步到氚云
            getMaterialFromU9(result.getContent());

        }
        log.info("更新U9物料数据结束,共更新数据"+totalAmount);
    }

    /**
     * 用于将需要更新的数据同步到氚云
     * @param toSave    待保存的数据集合
     */
    private void getMaterialFromU9(List<MaterialDTO> toSave){
        ObjectMapper objectMapper=new ObjectMapper();
        toSave.forEach(materialDTO -> {
           try {
               EquipmentMaterialDTO equipmentMaterialDTO = new EquipmentMaterialDTO(materialDTO);
               var materialString = objectMapper.writeValueAsString(equipmentMaterialDTO);
               List<String> projectmatchers = new ArrayList<>();
               projectmatchers.add("F0000002_2,"+materialDTO.getMaterialPrimaryKey().getCode());
               if (!StringUtils.isEmpty(materialDTO.getMaterialPrimaryKey().getVersion())){
                   projectmatchers.add("F0000010_2,"+materialDTO.getMaterialPrimaryKey().getVersion());
               }
               var projectfilter= Filter.instance(0,Integer.MAX_VALUE,true,"And",projectmatchers);
               var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMaterialDO.SCHEMA_CODE,projectfilter);
               ChuanyunSaveDTO chuanyunSaveDTO;
               if (chuanyunFindAllResponse.getReturnData()!=null){
                   List<ChuanyunMaterialDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                   chuanyunSaveDTO=chuanYunManager.delete(ChuanyunMaterialDO.SCHEMA_CODE,result.get(0).getObjectId());
               }
               chuanyunSaveDTO =chuanYunManager.save(ChuanyunMaterialDO.SCHEMA_CODE,materialString,true);
               if (chuanyunSaveDTO.getSuccessful()){
                   log.info("设备零部件更新成功");
               }else {
                   log.info("设备零部件更新失败");
               }
           } catch (JsonProcessingException | BigSizeException e) {
               e.printStackTrace();
           }
       });
    }

    /**
     * 定时更新项目信息
     */
    private void getProjectInformationFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter=Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                var objectMapper=new ObjectMapper();
                List<ChuanyunProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //更新缓存中的项目映射表
                result.forEach(projectBO -> ChuanYunConstant.PROJECT_CODE.put(projectBO.getProjectCode(),projectBO.getObjectId()));
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunProjectDO -> {
                    var tmpProjectDO=chuanyunProjectDao.findById(chuanyunProjectDO.getObjectId());
                    tmpProjectDO.ifPresent(projectDO -> chuanyunProjectDO.setIsCreate(projectDO.getIsCreate()));
                    //更新设备分类
                    if (chuanyunProjectDO.getEquipmentTypeList()!=null && chuanyunProjectDO.getEquipmentTypeList().length>0) {
                        var equipmentType=new StringBuilder();
                        for(int i=0;i<chuanyunProjectDO.getEquipmentTypeList().length;i++) {
                            var a = chuanyunProjectDO.getEquipmentTypeList()[i];
                            var chuanyunFindResponse =  chuanYunManager.find(ChuanyunEquipmentType.SCHEMA_CODE,chuanyunProjectDO.getEquipmentTypeList()[i]);
                            ChuanyunEquipmentType chuanyunEquipmentType =objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                            equipmentType.append(chuanyunEquipmentType.getEquipmentTypeName()).append(StringConstant.SEMICOLON);
                        }
                        chuanyunProjectDO.setEquipmentType(equipmentType.toString());
                    }
                    //将projectManger和projectOwner替换为主键
                    if(!StringUtils.isEmpty(chuanyunProjectDO.getProjectManager())){
                        var projectManagerIds=new StringBuilder();
                        for (ChuanyunObject object:chuanyunProjectDO.getProjectManagerObject()){
                            projectManagerIds.append(object.getObjectId()).append(StringConstant.SEMICOLON);
                        }
                        chuanyunProjectDO.setProjectManager(projectManagerIds.toString());
                    }
                    if(!StringUtils.isEmpty(chuanyunProjectDO.getProjectOwner())){
                        chuanyunProjectDO.setProjectOwner(chuanyunProjectDO.getProjectOwnerObject().getObjectId());
                    }
                });
                chuanyunProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    /**
     * 将物料补发数据同步到氚云
     * 预计2020年4月份之后逐步将各个部门物料补发数据迁移到氚云进行审批，这个功能逐步废弃
     * 不能调用这个方法！！！！！
     */
    @Deprecated
    private void putMaterialResendInstance(){
        var currentPage=0;
        var totalPage=Integer.MAX_VALUE;

        var objectMapper=new ObjectMapper();
        objectMapper.registerModule(new JavaTimeModule());
        objectMapper.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);

        while (currentPage<totalPage){
            List<String> strings=new ArrayList<>();
            PageRequest pageRequest=PageRequest.of(currentPage,pageSize);
            DingTalkProcessInstance dingTalkProcessInstance=new DingTalkProcessInstance();
            dingTalkProcessInstance.setProcessInstanceId(DingTalkConstant.PROCESS_CODE_MATERIAL_RESEND);
            dingTalkProcessInstance.setUpdatedInChuanyun(false);
            Page<DingTalkProcessInstance> pageList=dingTalkProcessInstanceDao.findAll(Example.of(dingTalkProcessInstance),pageRequest);
            if(totalPage==Integer.MAX_VALUE){
                totalPage=pageList.getTotalPages();
            }

            pageList.getContent().forEach(content->{
                if(content.getProcessInstance()==null){
                    log.info("观察异常内容"+content+"");
                }else {
                    if (content.getProcessInstanceId().equals(DingTalkConstant.PROCESS_CODE_MATERIAL_RESEND)){
                        MaterialResendBO materialResendBO = MaterialResendBO.fromDingTak(content.getProcessInstance());
                        try {
                            strings.add(objectMapper.writeValueAsString(materialResendBO));
                        } catch (JsonProcessingException e) {
                            log.error(e.getMessage());
                        }
                    }
                }
            });
            try{
                ChuanyunSaveAllDTO chuanyunSaveAllResponse= chuanYunManager.saveAll(MaterialResendBO.SCHEMA_CODE,strings,true);
                log.info("氚云保存物料补发数据结果的错误信息为 "+chuanyunSaveAllResponse.getErrorMessage());
                log.info(chuanyunSaveAllResponse+"");
                pageList.getContent().forEach(content->content.setUpdatedInChuanyun(true));
                log.info("开始更新钉钉审批数据上传状态");
                log.info(dingTalkProcessInstanceDao.saveAll(pageList.getContent())+"");
                log.info("已更新上传状态的钉钉审批数据");
            }catch (Exception e){
                e.printStackTrace();
            }
            currentPage++;
        }
    }

    @Data
    class EquipmentMaterialDTO{

        @JsonProperty(value = "F0000002")
        private String code;

        @JsonProperty(value = "F0000001")
        private String title;

        @JsonProperty(value = "F0000007")
        private String label;

        @JsonProperty(value = "F0000008")
        private String model;

        @JsonProperty(value = "F0000009")
        private String brand;

        @JsonProperty(value = "F0000010")
        private String version;

        @JsonProperty(value = "F0000005")
        private String type;

        @JsonProperty(value = "F0000006")
        private String property;

        public EquipmentMaterialDTO(MaterialDTO materialDTO){
            if (materialDTO!=null){
                if (materialDTO.getMaterialPrimaryKey().getCode()!=null){
                    this.code=materialDTO.getMaterialPrimaryKey().getCode();
                }
                if (materialDTO.getMaterialPrimaryKey().getVersion()!=null){
                    this.version=materialDTO.getMaterialPrimaryKey().getVersion();
                }
                if (materialDTO.getName()!=null){
                    this.title=materialDTO.getName();
                }
                if (materialDTO.getBrand()!=null){
                    this.brand=materialDTO.getBrand();
                }
                if (materialDTO.getLabel()!=null){
                    this.label=materialDTO.getLabel();
                }
                if (materialDTO.getType()!=null){
                    this.type=materialDTO.getType();
                }
                if (materialDTO.getProperty()!=null){
                    this.property=materialDTO.getProperty();
                }
            }
        }
    }
}
