package com.galaxis.manatee.task;

import com.galaxis.manatee.dao.ChuanyunGroupProjectDao;
import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.service.LogCheckService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

/**
 * @author zcj
 * @version 0.1
 * @date 2021/3/4 5:19 上午
 */
@Slf4j
@Service
public class LogCheckTask {

    private final LogCheckService logCheckService;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;

    public LogCheckTask(LogCheckService logCheckService, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao) {
        this.logCheckService = logCheckService;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
    }

    /**
     * 定时任务每5天执行一次
     * 定时进行日志工时核对任务
     * 1.遍历每个用户，核对原始数据和标准化每日工时
     * 2.遍历每个用户，核对每周、每月工时准确性
     * 3.遍历每个项目核对每个项目总工时
     */
//    @Scheduled(fixedDelay = 432000000L)
    private void checkLod(){
        chuanyunUserCompanyDao.findAll().forEach(chuanyunUserCompanyDO -> {
            long start=System.currentTimeMillis();
            try {
                //核对原始数据和标准化每日工时
                logCheckService.checkDayWorkHour(chuanyunUserCompanyDO.getUserId());
                logCheckService.checkWeekWorkHour(chuanyunUserCompanyDO.getUserId());
                logCheckService.checkMonthWorkHour(chuanyunUserCompanyDO.getUserId());
            } catch (Exception e) {
                log.error(""+chuanyunUserCompanyDO);
                log.error(e.getMessage());
            }
            log.info("修改"+chuanyunUserCompanyDO.getUserName()+"工时信息耗时"+(System.currentTimeMillis()-start));
        });
        //核对每周、每月、每个项目工时准确性
        chuanyunGroupProjectDao.findAll().forEach(chuanyunGroupProjectDO -> {
            try{
                logCheckService.checkProjectWorkHour(chuanyunGroupProjectDO.getObjectId());
            }catch (Exception e){
                log.error(""+chuanyunGroupProjectDO);
                log.error(e.getMessage());
            }
        });
    }
}
