package com.galaxis.manatee.task;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunDayWorkAbcDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunDayWorkAbcDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunDayWorkAbcDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/8/9 11:01 下午
 */
@Service
@Slf4j
public class WorkHourAnalysis {

    private final ChuanyunDayWorkAbcDao chuanyunDayWorkAbcDao;

    private final ChuanYunManager chuanYunManager;

    public WorkHourAnalysis(ChuanyunDayWorkAbcDao chuanyunDayWorkAbcDao,ChuanYunManager chuanYunManager) {
        this.chuanyunDayWorkAbcDao = chuanyunDayWorkAbcDao;
        this.chuanYunManager = chuanYunManager;
    }

    /**
     * ABC工时分析暂时不用
     */
    @Deprecated
    private void standardDayWorkAbcHour() {
        standardDayAbcHour();
           // standardChuanyunDayWorkAbcHour();
    }

    private void standardChuanyunDayWorkAbcHour() {
        var objectMapper = new ObjectMapper();
        List<ChuanyunDayWorkAbcDO> dayWorkAbcDOList = chuanyunDayWorkAbcDao.findAll();
        dayWorkAbcDOList.forEach(dayWork -> {
            try {
                var putString = objectMapper.writeValueAsString(ChuanyunDayWorkAbcDTO.fromChuanyunDayWorkAbcDTO(dayWork));
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000001_2," + dayWork.getProjectCode());
                var filter = Filter.instance(0, 1, true, "And", matchers);
                ChuanyunFindAllBizDTO<LinkedHashMap<String, String>> chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunDayWorkAbcDO.SCHEMA_CODE, filter);
                if(chuanyunFindAllResponse.getReturnData() == null) {
                    chuanYunManager.save(ChuanyunDayWorkAbcDO.SCHEMA_CODE,putString,true);
                }else{
                    chuanYunManager.update(ChuanyunDayWorkAbcDO.SCHEMA_CODE,chuanyunFindAllResponse.getReturnData().getBizObjectArray().get(0).get("ObjectId"),putString);
                }
            }catch(Exception e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 员工工时abc分析
     */
    private void standardDayAbcHour() {
        log.info("开始更新工时abc分析数据");
        long startTime= Instant.now().getEpochSecond();
        List<Map<String, Object>> dayWorkAbcList = chuanyunDayWorkAbcDao.queryDayWorkAbc();
        dayWorkAbcList.forEach(dayWork -> {
            try {
                String projectId = (String)dayWork.get("project_id");
                String projectName = (String)dayWork.get("project_name");
                String projectCode = (String)dayWork.get("project_code");
                BigDecimal totalTime = (BigDecimal)dayWork.get("total_time");
                LocalDateTime dayLogDate = ((Timestamp)dayWork.get("day_log_date")).toLocalDateTime();
                ChuanyunDayWorkAbcDO dayWorkAbcDO = new ChuanyunDayWorkAbcDO();
                ChuanyunDayWorkAbcDO planDayWorkAbc = chuanyunDayWorkAbcDao.queryDayWorkAbcTotalTime(projectId,0);
                if(planDayWorkAbc != null) {
                    dayWorkAbcDO.setPlanTime(planDayWorkAbc.getTotalTime());
                }
                ChuanyunDayWorkAbcDO sellDayWorkAbc = chuanyunDayWorkAbcDao.queryDayWorkAbcTotalTime(projectId,1);
                if(sellDayWorkAbc != null) {
                    dayWorkAbcDO.setSellTime(sellDayWorkAbc.getTotalTime());
                }
                ChuanyunDayWorkAbcDO otherWorkAbc = chuanyunDayWorkAbcDao.queryDayWorkAbcTotalTime(projectId,2);
                if(otherWorkAbc != null) {
                    dayWorkAbcDO.setOtherTime(otherWorkAbc.getTotalTime());
                }
                ChuanyunDayWorkAbcDO chuanyunDayWorkAbcDO = chuanyunDayWorkAbcDao.findByProjectId(projectId);
                if (chuanyunDayWorkAbcDO != null) {
                    chuanyunDayWorkAbcDao.delete(chuanyunDayWorkAbcDO);
                }
                dayWorkAbcDO.setObjectId(UUID.randomUUID().toString());
                dayWorkAbcDO.setProjectId(projectId);
                dayWorkAbcDO.setProjectCode(projectCode);
                dayWorkAbcDO.setProjectName(projectName);
                dayWorkAbcDO.setTotalTime(totalTime);
                dayWorkAbcDO.setDayLogDate(dayLogDate);
                chuanyunDayWorkAbcDao.save(dayWorkAbcDO);
            }catch(Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        });

        log.info("周工时abc分析耗时"+(Instant.now().getEpochSecond()-startTime)+"秒");
    }
}
